# -*- coding: utf-8 -*-
"""
sub_code_agent 工具
将子任务交给 CodeAgent 执行，并返回执行结果。

约定：
- 必填参数：task
- 可选参数：background
- 不依赖父 Agent，所有配置使用系统默认与全局变量
- 子Agent必须自动完成(auto_complete=True)且需要summary(need_summary=True)
"""

from typing import Any, Dict, List

from jarvis.jarvis_code_agent.code_agent import CodeAgent
from jarvis.jarvis_utils.globals import get_global_model_group


class SubCodeAgentTool:
    """
    使用 CodeAgent 托管执行子任务，执行完立即清理 CodeAgent 实例。
    - 不注册至全局
    - 使用系统默认/全局配置
    - 启用自动完成与总结
    """

    # 必须与文件名一致，供 ToolRegistry 自动注册
    name = "sub_code_agent"
    description = "将子任务交给 CodeAgent 执行并返回结果（自动完成并生成总结）。"
    parameters = {
        "type": "object",
        "properties": {
            "task": {
                "type": "string",
                "description": "要执行的子任务内容（必填）",
            },
            "background": {
                "type": "string",
                "description": "任务背景与已知信息（可选，将与任务一并提供给子Agent）",
            },
            "name": {
                "type": "string",
                "description": "子Agent的名称（可选，用于标识和区分不同的子Agent）",
            },
        },
        "required": ["task"],
    }

    def execute(self, args: Dict[str, Any]) -> Dict[str, Any]:
        """
        执行代码子任务并返回结果（由 CodeAgent 托管执行）。
        返回:
          - success: 是否成功
          - stdout: CodeAgent 执行结果（字符串；若为 None 则返回“任务执行完成”）
          - stderr: 错误信息（如有）
        """
        try:
            task: str = str(args.get("task", "")).strip()
            if not task:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "task 不能为空",
                }

            # 读取背景信息并组合任务
            background: str = str(args.get("background", "")).strip()
            enhanced_task = (
                f"背景信息:\n{background}\n\n任务:\n{task}" if background else task
            )

            # 读取子Agent名称（可选）
            agent_name: str = str(args.get("name", "")).strip()

            # 继承父Agent的模型组与工具使用集（用于覆盖默认值）
            parent_agent = args.get("agent")
            # 如未注入父Agent，尝试从全局获取当前或任一已注册Agent
            if parent_agent is None:
                try:
                    from jarvis.jarvis_utils import globals as G  # 延迟导入避免循环

                    curr = G.get_current_agent_name()
                    if curr:
                        parent_agent = getattr(G, "global_agents", {}).get(curr)
                    if parent_agent is None and getattr(G, "global_agents", {}):
                        try:
                            parent_agent = next(iter(G.global_agents.values()))
                        except Exception:
                            parent_agent = None
                except Exception:
                    parent_agent = None
            (
                getattr(parent_agent, "non_interactive", None)
                if parent_agent is not None
                else None
            )
            # 使用全局模型组（不再从 parent_agent 继承）
            model_group = get_global_model_group()
            use_tools: List[str] = []
            try:
                if parent_agent is not None:
                    parent_registry = parent_agent.get_tool_registry()
                    if parent_registry:
                        for t in parent_registry.get_all_tools():
                            if isinstance(t, dict) and t.get("name"):
                                use_tools.append(str(t["name"]))
            except Exception:
                pass

            # 创建 CodeAgent：参数优先使用父Agent的配置（若可获取），否则使用默认
            # 推断/继承 tool_group
            tool_group = None
            try:
                if parent_agent is not None:
                    tool_group = getattr(parent_agent, "tool_group", tool_group)
            except Exception:
                pass

            # 依据父Agent已启用工具集，推导 append_tools（作为在 CodeAgent 基础工具上的增量）
            # 禁用 sub_agent 和 sub_code_agent，避免无限递归
            forbidden_tools = {"sub_agent", "sub_code_agent"}
            append_tools = None
            try:
                base_tools = [
                    "execute_script",
                    "read_code",
                    "edit_file",
                ]
                if use_tools:
                    # 过滤掉基础工具和禁止的工具
                    extras = [
                        t
                        for t in use_tools
                        if t not in base_tools and t not in forbidden_tools
                    ]
                    append_tools = ",".join(extras) if extras else None
            except Exception:
                append_tools = None

            # 获取父代理的规则名称列表用于继承
            rule_names = None
            try:
                if parent_agent is not None and hasattr(
                    parent_agent, "loaded_rule_names"
                ):
                    parent_rules = getattr(parent_agent, "loaded_rule_names", [])
                    if parent_rules:
                        rule_names = ",".join(parent_rules)
            except Exception:
                rule_names = None

            # 自动禁用review功能
            disable_review = True

            # 创建 CodeAgent，捕获 SystemExit 异常（如 git 配置不完整）
            try:
                code_agent = CodeAgent(
                    name=agent_name,
                    model_group=model_group,
                    need_summary=True,
                    append_tools=append_tools,
                    tool_group=tool_group,
                    non_interactive=True,
                    rule_names=rule_names,
                    disable_review=disable_review,
                    auto_complete=True,
                )
            except SystemExit as se:
                # 将底层 sys.exit 转换为工具错误，避免终止进程
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": f"初始化 CodeAgent 失败（可能未配置 git 或当前非 git 仓库）: {se}",
                }

            # 子Agent需要自动完成
            try:
                # 同步父Agent工具使用集（如可用），但禁用 sub_agent 和 sub_code_agent 避免无限递归
                if use_tools:
                    forbidden_tools = {"sub_agent", "sub_code_agent"}
                    filtered_tools = [t for t in use_tools if t not in forbidden_tools]
                    if filtered_tools:
                        code_agent.set_use_tools(filtered_tools)
                # 不再从父Agent获取模型名，使用系统默认配置（符合"不依赖父 Agent"的约定）
            except Exception:
                pass

            # 执行子任务（无提交信息前后缀）
            ret = code_agent.run(enhanced_task, prefix="", suffix="")

            return {
                "success": True,
                "stdout": ret,
                "stderr": "",
            }
        except Exception as e:
            return {
                "success": False,
                "stdout": "",
                "stderr": f"执行子任务失败: {str(e)}",
            }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
自定义构建验证器模块

支持用户在配置文件中定义自定义构建命令进行验证。
"""

import time
from typing import List
from typing import Optional

from jarvis.jarvis_utils.output import PrettyOutput

from .base import BuildResult
from .base import BuildSystem
from .base import BuildValidatorBase


class CustomBuildValidator(BuildValidatorBase):
    """自定义构建验证器"""

    BUILD_SYSTEM_NAME = "Custom"
    SUPPORTED_LANGUAGES = ["*"]  # 支持所有语言

    def __init__(
        self, project_root: str, timeout: int = 30, command: Optional[str] = None
    ):
        super().__init__(project_root, timeout)
        self.custom_command = command
        # 导入配置管理器
        from jarvis.jarvis_code_agent.build_validation_config import (
            BuildValidationConfig,
        )

        self.config = BuildValidationConfig(project_root)
        # 如果没有传入命令，从配置文件读取
        if self.custom_command is None:
            self.custom_command = self.config.get_custom_build_command()

    def validate(self, modified_files: Optional[List[str]] = None) -> BuildResult:
        """验证构建

        Args:
            modified_files: 修改的文件列表（此验证器不使用此参数）

        Returns:
            BuildResult: 验证结果
        """
        start_time = time.time()

        # 检查是否配置了自定义命令
        if not self.custom_command:
            duration = time.time() - start_time
            error_msg = "未配置自定义构建命令，请在 .jarvis/build_validation_config.yaml 中设置 custom_build_command"
            PrettyOutput.auto_print(f"❌ {error_msg}")
            return BuildResult(
                success=False,
                output="",
                error_message=error_msg,
                build_system=BuildSystem.CUSTOM,
                duration=duration,
            )

        PrettyOutput.auto_print(f"🔧 执行自定义构建命令: {self.custom_command}")

        # 解析命令（支持复杂命令，如 "make && make test"）
        try:
            # 使用 bash 执行命令，以支持 shell 特性（如 &&、||、管道等）
            returncode, stdout, stderr = self._run_command(
                ["bash", "-c", self.custom_command],
                cwd=self.project_root,
                timeout=self.timeout,
                capture_output=True,
            )
        except Exception as e:
            duration = time.time() - start_time
            error_msg = f"执行自定义构建命令时出错: {str(e)}"
            PrettyOutput.auto_print(f"❌ {error_msg}")
            return BuildResult(
                success=False,
                output="",
                error_message=error_msg,
                build_system=BuildSystem.CUSTOM,
                duration=duration,
            )

        duration = time.time() - start_time
        success = returncode == 0
        output = stdout + stderr

        if success:
            PrettyOutput.auto_print(f"✅ 自定义构建验证成功（耗时 {duration:.2f} 秒）")
        else:
            PrettyOutput.auto_print(
                f"❌ 自定义构建验证失败（耗时 {duration:.2f} 秒，返回码: {returncode}）"
            )
            # 显示输出（限制长度）
            output_preview = output[:1000] if len(output) > 1000 else output
            PrettyOutput.auto_print(f"输出：\n{output_preview}")
            if len(output) > 1000:
                PrettyOutput.auto_print(
                    f"...（输出已截断，完整输出共 {len(output)} 字符）"
                )

        return BuildResult(
            success=success,
            output=output,
            error_message=None
            if success
            else f"自定义构建命令返回错误码: {returncode}",
            build_system=BuildSystem.CUSTOM,
            duration=duration,
        )



"""CodeAgent 规则管理模块"""

import os
import subprocess

from jarvis.jarvis_utils.output import PrettyOutput

# -*- coding: utf-8 -*-
from typing import List
from typing import Optional
from typing import Set
from typing import Tuple

import yaml

from jarvis.jarvis_agent.builtin_rules import get_builtin_rule
from jarvis.jarvis_utils.template_utils import render_rule_template
from jarvis.jarvis_utils.config import get_central_rules_repo
from jarvis.jarvis_utils.config import get_data_dir
from jarvis.jarvis_utils.config import get_rules_load_dirs
from jarvis.jarvis_utils.utils import daily_check_git_updates


class RulesManager:
    """规则管理器，负责加载和管理各种规则"""

    def __init__(self, root_dir: str):
        self.root_dir = root_dir
        # 初始化规则目录列表
        self._init_rules_dirs()

    def _init_rules_dirs(self) -> None:
        """初始化规则目录列表，包括配置的目录和中心库"""
        # 基础目录：全局数据目录下的 rules 目录
        self.rules_dirs: List[str] = [os.path.join(get_data_dir(), "rules")]

        # 添加配置的规则加载目录
        self.rules_dirs.extend(get_rules_load_dirs())

        # 中心规则仓库路径（单独存储，优先级最高）
        self.central_repo_path: Optional[str] = None
        central_repo = get_central_rules_repo()
        if central_repo:
            # 支持本地目录路径或Git仓库URL
            expanded = os.path.expanduser(os.path.expandvars(central_repo))
            if os.path.isdir(expanded):
                # 直接使用本地目录（支持Git仓库的子目录）
                self.central_repo_path = expanded
            else:
                # 中心规则仓库存储在数据目录下的特定位置
                self.central_repo_path = os.path.join(
                    get_data_dir(), "central_rules_repo"
                )

                # 确保中心规则仓库被克隆/更新
                if not os.path.exists(self.central_repo_path):
                    try:
                        PrettyOutput.auto_print(
                            f"ℹ️ 正在克隆中心规则仓库: {central_repo}"
                        )
                        subprocess.run(
                            ["git", "clone", central_repo, self.central_repo_path],
                            check=True,
                        )
                    except Exception as e:
                        PrettyOutput.auto_print(f"❌ 克隆中心规则仓库失败: {str(e)}")

        # 执行每日更新检查（包括中心库）
        all_dirs_for_update = self.rules_dirs.copy()
        if self.central_repo_path:
            all_dirs_for_update.append(self.central_repo_path)
        daily_check_git_updates(all_dirs_for_update, "rules")

    def read_project_rule(self) -> Optional[str]:
        """读取 .jarvis/rule 文件内容，如果存在则返回字符串，否则返回 None"""
        try:
            rules_path = os.path.join(self.root_dir, ".jarvis", "rule")
            if os.path.exists(rules_path) and os.path.isfile(rules_path):
                with open(rules_path, "r", encoding="utf-8", errors="replace") as f:
                    content = f.read().strip()
                # 使用jinja2渲染规则模板
                if content:
                    content = render_rule_template(content, os.path.dirname(rules_path))
                return content if content else None
        except Exception:
            # 读取规则失败时忽略，不影响主流程
            pass
        return None

    def read_global_rules(self) -> Optional[str]:
        """读取数据目录 rules 内容，如果存在则返回字符串，否则返回 None"""
        try:
            rules_path = os.path.join(get_data_dir(), "rule")
            if os.path.exists(rules_path) and os.path.isfile(rules_path):
                with open(rules_path, "r", encoding="utf-8", errors="replace") as f:
                    content = f.read().strip()
                # 使用jinja2渲染规则模板
                if content:
                    content = render_rule_template(content, os.path.dirname(rules_path))
                return content if content else None
        except Exception:
            # 读取规则失败时忽略，不影响主流程
            pass
        return None

    def _read_rule_from_dir(self, rules_dir: str, rule_name: str) -> Optional[str]:
        """从 rules 目录中读取指定名称的规则文件

        参数:
            rules_dir: rules 目录路径
            rule_name: 规则名称（支持相对路径，如 deployment/version_release.md）

        返回:
            str: 规则内容，如果未找到则返回 None
        """
        try:
            # 只支持 .md 后缀的文件
            if not rule_name.endswith(".md"):
                rule_name = rule_name + ".md"
            # 支持相对路径（如 deployment/version_release.md）
            rule_file_path = os.path.join(rules_dir, rule_name)
            if os.path.exists(rule_file_path) and os.path.isfile(rule_file_path):
                with open(rule_file_path, "r", encoding="utf-8", errors="replace") as f:
                    content = f.read().strip()
                # 使用jinja2渲染规则模板
                if content:
                    # 使用规则文件所在目录作为模板渲染的上下文
                    content = render_rule_template(
                        content, os.path.dirname(rule_file_path)
                    )
                return content if content else None
        except Exception:
            # 读取规则失败时忽略，不影响主流程
            pass
        return None

    def _get_builtin_rules_index(self) -> Optional[str]:
        """读取 builtin_rules.md 索引文件的完整内容

        返回:
            str: builtin_rules.md 的完整内容，如果未找到则返回 None
        """
        try:
            from jarvis.jarvis_utils.template_utils import _get_jarvis_src_dir

            # 获取索引文件路径
            jarvis_src_dir = _get_jarvis_src_dir()
            index_file_path = os.path.join(
                jarvis_src_dir, "builtin", "rules", "builtin_rules.md"
            )

            # 检查索引文件是否存在
            if not os.path.exists(index_file_path) or not os.path.isfile(
                index_file_path
            ):
                return None

            # 读取索引文件内容
            with open(index_file_path, "r", encoding="utf-8", errors="replace") as f:
                index_content = f.read()

            # 使用jinja2渲染规则模板
            if index_content:
                index_content = render_rule_template(
                    index_content, os.path.dirname(index_file_path)
                )

            return index_content if index_content else None

        except Exception as e:
            # 读取失败时忽略，不影响主流程
            PrettyOutput.auto_print(f"⚠️ 读取builtin_rules.md失败: {e}")
            return None

    def _get_rule_from_builtin_index(self, rule_name: str) -> Optional[str]:
        """从 builtin_rules.md 索引文件中查找并加载指定名称的规则

        该索引文件记录了内置规则的映射关系，格式为：
        - [规则名称]({{ template_var }}/path/to/rule.md)

        参数:
            rule_name: 规则名称

        返回:
            str: 规则内容，如果未找到则返回 None
        """
        try:
            from jarvis.jarvis_utils.template_utils import _get_jarvis_src_dir

            # 获取索引文件路径
            jarvis_src_dir = _get_jarvis_src_dir()
            index_file_path = os.path.join(
                jarvis_src_dir, "builtin", "rules", "builtin_rules.md"
            )

            # 检查索引文件是否存在
            if not os.path.exists(index_file_path) or not os.path.isfile(
                index_file_path
            ):
                return None

            # 读取索引文件内容
            with open(index_file_path, "r", encoding="utf-8", errors="replace") as f:
                index_content = f.read()

            # 解析索引文件，查找匹配的规则
            # 格式: - [规则名称](路径)
            import re

            pattern = rf"-\s*\[{re.escape(rule_name)}\]\(([^)]+)\)"
            match = re.search(pattern, index_content)

            if not match:
                return None

            # 提取规则文件路径
            rule_file_template = match.group(1).strip()

            # 渲染模板变量（支持 {{ jarvis_src_dir }} 和 {{ rule_file_dir }}）
            context = {
                "jarvis_src_dir": jarvis_src_dir,
                "rule_file_dir": os.path.dirname(index_file_path),
            }

            try:
                from jinja2 import Template

                template = Template(rule_file_template)
                rule_file_path = template.render(**context)
            except Exception:
                # 模板渲染失败，直接使用原始路径
                rule_file_path = rule_file_template

            # 检查规则文件是否存在
            if not os.path.exists(rule_file_path) or not os.path.isfile(rule_file_path):
                return None

            # 读取规则文件内容
            with open(rule_file_path, "r", encoding="utf-8", errors="replace") as f:
                rule_content = f.read().strip()

            # 使用jinja2渲染规则模板
            if rule_content:
                rule_content = render_rule_template(
                    rule_content, os.path.dirname(rule_file_path)
                )

            return rule_content if rule_content else None

        except Exception as e:
            # 读取失败时忽略，不影响主流程
            PrettyOutput.auto_print(f"⚠️ 从索引文件加载规则失败: {e}")
            return None

    def _get_all_rules_dirs(self) -> List[str]:
        """获取所有规则目录（包括项目目录和配置的目录）

        返回:
            List[str]: 规则目录列表，按优先级排序（中心库 > 项目 > 配置目录）
        """
        all_dirs = []
        # 优先级 1: 中心规则仓库（如果有同名规则，以中心仓库为准）
        if (
            self.central_repo_path
            and os.path.exists(self.central_repo_path)
            and os.path.isdir(self.central_repo_path)
        ):
            # 检查中心仓库中是否有 rules 子目录
            central_rules_dir = os.path.join(self.central_repo_path, "rules")
            if os.path.exists(central_rules_dir) and os.path.isdir(central_rules_dir):
                all_dirs.append(central_rules_dir)
            else:
                # 如果没有 rules 子目录，直接使用中心仓库根目录
                all_dirs.append(self.central_repo_path)
        # 优先级 2: 项目 rules 目录
        project_rules_dir = os.path.join(self.root_dir, ".jarvis", "rules")
        if os.path.exists(project_rules_dir) and os.path.isdir(project_rules_dir):
            all_dirs.append(project_rules_dir)
        # 优先级 3-N: 配置的规则目录（不包括中心库）
        all_dirs.extend(self.rules_dirs)
        return all_dirs

    def _get_all_rules_yaml_files(self) -> List[tuple[str, str]]:
        """获取所有 rules.yaml 文件路径（描述，文件路径）

        返回:
            List[tuple[str, str]]: (描述, 文件路径) 列表，按优先级排序（中心库 > 项目 > 全局）
        """
        yaml_files = []
        # 优先级 1: 中心规则仓库的 rules.yaml（如果有同名规则，以中心仓库为准）
        if self.central_repo_path and os.path.exists(self.central_repo_path):
            central_rules_yaml = os.path.join(self.central_repo_path, "rules.yaml")
            if os.path.exists(central_rules_yaml) and os.path.isfile(
                central_rules_yaml
            ):
                yaml_files.append(("中心库", central_rules_yaml))
        # 优先级 2: 项目 rules.yaml
        project_rules_yaml = os.path.join(self.root_dir, ".jarvis", "rules.yaml")
        if os.path.exists(project_rules_yaml) and os.path.isfile(project_rules_yaml):
            yaml_files.append(("项目", project_rules_yaml))
        # 优先级 3: 全局 rules.yaml
        global_rules_yaml = os.path.join(get_data_dir(), "rules.yaml")
        if os.path.exists(global_rules_yaml) and os.path.isfile(global_rules_yaml):
            yaml_files.append(("全局", global_rules_yaml))
        return yaml_files

    def get_named_rule(self, rule_name: str) -> Optional[str]:
        """从 rules.yaml 文件和 rules 目录中获取指定名称的规则

        规则名称格式：前缀:规则名
        前缀说明：
        - builtin: - 内置规则目录（jarvis源码/builtin/rules）
        - project: - 项目 .jarvis/rules 目录文件
        - global: - 全局 ~/.jarvis/rules 目录文件
        - central: - 中心规则仓库文件
        - config1:, config2: - 配置的规则目录文件
        - central_yaml: - 中心库 rules.yaml
        - project_yaml: - 项目 rules.yaml
        - global_yaml: - 全局 rules.yaml
        - 无前缀 - 内置规则或 builtin_rules.md 索引文件中的规则

        参数:
            rule_name: 规则名称（可能包含前缀）

        返回:
            str: 规则内容，如果未找到则返回 None
        """
        try:
            # 解析前缀
            if ":" in rule_name:
                prefix, actual_name = rule_name.split(":", 1)
                if not actual_name:
                    return None

                # 处理 builtin 前缀
                if prefix == "builtin":
                    try:
                        from jarvis.jarvis_utils.template_utils import (
                            _get_jarvis_src_dir,
                        )

                        jarvis_src_dir = _get_jarvis_src_dir()
                        builtin_rules_dir = os.path.join(
                            jarvis_src_dir, "builtin", "rules"
                        )
                        if os.path.exists(builtin_rules_dir) and os.path.isdir(
                            builtin_rules_dir
                        ):
                            return self._read_rule_from_dir(
                                builtin_rules_dir, actual_name
                            )
                    except Exception:
                        pass
                    return None

                # 处理 project 前缀
                if prefix == "project":
                    project_rules_dir = os.path.join(self.root_dir, ".jarvis", "rules")
                    if os.path.exists(project_rules_dir) and os.path.isdir(
                        project_rules_dir
                    ):
                        return self._read_rule_from_dir(project_rules_dir, actual_name)
                    return None

                # 处理 global 前缀
                if prefix == "global":
                    global_rules_dir = os.path.join(get_data_dir(), "rules")
                    if os.path.exists(global_rules_dir) and os.path.isdir(
                        global_rules_dir
                    ):
                        return self._read_rule_from_dir(global_rules_dir, actual_name)
                    return None

                # 处理 central 和 config 前缀
                if prefix == "central" or prefix.startswith("config"):
                    all_rules_dirs = self._get_all_rules_dirs()
                    target_idx = -1
                    if prefix == "central" and len(all_rules_dirs) > 0:
                        target_idx = 0
                    elif prefix.startswith("config"):
                        try:
                            config_num = int(prefix[6:])
                            target_idx = 2 + config_num
                        except ValueError:
                            pass

                    if 0 <= target_idx < len(all_rules_dirs):
                        rules_dir = all_rules_dirs[target_idx]
                        if os.path.exists(rules_dir) and os.path.isdir(rules_dir):
                            return self._read_rule_from_dir(rules_dir, actual_name)
                    return None

                # 处理 yaml 规则
                elif prefix in ["central_yaml", "project_yaml", "global_yaml"]:
                    for desc, yaml_path in self._get_all_rules_yaml_files():
                        if (prefix == "central_yaml" and desc == "中心库") or (
                            prefix == "project_yaml" and desc == "项目"
                        ):
                            if os.path.exists(yaml_path) and os.path.isfile(yaml_path):
                                try:
                                    with open(
                                        yaml_path,
                                        "r",
                                        encoding="utf-8",
                                        errors="replace",
                                    ) as f:
                                        rules = yaml.safe_load(f) or {}
                                    if actual_name in rules:
                                        rule_value = rules[actual_name]
                                        if isinstance(rule_value, str):
                                            content = rule_value.strip()
                                        else:
                                            content = str(rule_value).strip()
                                        # 使用jinja2渲染规则模板
                                        if content:
                                            content = render_rule_template(
                                                content, os.path.dirname(yaml_path)
                                            )
                                        return content if content else None
                                except Exception:
                                    continue
                        elif prefix == "global_yaml" and desc == "全局":
                            if os.path.exists(yaml_path) and os.path.isfile(yaml_path):
                                try:
                                    with open(
                                        yaml_path,
                                        "r",
                                        encoding="utf-8",
                                        errors="replace",
                                    ) as f:
                                        rules = yaml.safe_load(f) or {}
                                    if actual_name in rules:
                                        rule_value = rules[actual_name]
                                        if isinstance(rule_value, str):
                                            content = rule_value.strip()
                                        else:
                                            content = str(rule_value).strip()
                                        # 使用jinja2渲染规则模板
                                        if content:
                                            content = render_rule_template(
                                                content, os.path.dirname(yaml_path)
                                            )
                                        return content if content else None
                                except Exception:
                                    continue
                    return None

                # 未知前缀
                return None

            # 无前缀：按优先级查找（项目 rules.yaml > 全局 rules.yaml > builtin_rules.md > 内置规则）
            # 优先级 1: 从项目 rules.yaml 文件中查找
            for desc, yaml_path in self._get_all_rules_yaml_files():
                if desc == "项目":
                    if os.path.exists(yaml_path) and os.path.isfile(yaml_path):
                        try:
                            with open(
                                yaml_path,
                                "r",
                                encoding="utf-8",
                                errors="replace",
                            ) as f:
                                rules = yaml.safe_load(f) or {}
                            if rule_name in rules:
                                rule_value = rules[rule_name]
                                if isinstance(rule_value, str):
                                    content = rule_value.strip()
                                else:
                                    content = str(rule_value).strip()
                                # 使用jinja2渲染规则模板
                                if content:
                                    content = render_rule_template(
                                        content, os.path.dirname(yaml_path)
                                    )
                                if content:
                                    return content
                        except Exception:
                            continue
            
            # 优先级 2: 从全局 rules.yaml 文件中查找
            for desc, yaml_path in self._get_all_rules_yaml_files():
                if desc == "全局":
                    if os.path.exists(yaml_path) and os.path.isfile(yaml_path):
                        try:
                            with open(
                                yaml_path,
                                "r",
                                encoding="utf-8",
                                errors="replace",
                            ) as f:
                                rules = yaml.safe_load(f) or {}
                            if rule_name in rules:
                                rule_value = rules[rule_name]
                                if isinstance(rule_value, str):
                                    content = rule_value.strip()
                                else:
                                    content = str(rule_value).strip()
                                # 使用jinja2渲染规则模板
                                if content:
                                    content = render_rule_template(
                                        content, os.path.dirname(yaml_path)
                                    )
                                if content:
                                    return content
                        except Exception:
                            continue
            
            # 优先级 3: 从 builtin_rules.md 索引文件中查找
            indexed_rule = self._get_rule_from_builtin_index(rule_name)
            if indexed_rule:
                return indexed_rule

            # 优先级 4: 从内置规则中查找
            builtin_rule = get_builtin_rule(rule_name)
            if builtin_rule:
                return builtin_rule

            return None
        except Exception as e:
            # 读取规则失败时忽略，不影响主流程
            PrettyOutput.auto_print(f"⚠️ 读取规则失败: {e}")
            return None

    def get_all_available_rule_names(self) -> dict[str, List[str]]:
        """获取所有可用的规则名称，按来源分类

        返回:
            dict[str, List[str]]: 按来源分类的规则名称字典
                - "builtin": 内置规则列表
                - "files": 规则目录中的文件规则列表（带来源前缀）
                - "yaml": rules.yaml 文件中的规则列表（带来源前缀）
        """
        from jarvis.jarvis_agent.builtin_rules import list_builtin_rules

        result = {
            "builtin": list_builtin_rules(),
            "files": [],
            "yaml": [],
        }

        # 收集规则目录中的文件规则（支持递归遍历子目录）
        all_rules_dirs = self._get_all_rules_dirs()
        for idx, rules_dir in enumerate(all_rules_dirs):
            if os.path.exists(rules_dir) and os.path.isdir(rules_dir):
                # 确定来源前缀（根据实际来源动态判断）
                # 检查是否为中心规则仓库
                is_central = False
                if self.central_repo_path:
                    central_rules_dir = os.path.join(self.central_repo_path, "rules")
                    if (
                        rules_dir == central_rules_dir
                        or rules_dir == self.central_repo_path
                    ):
                        is_central = True

                # 检查是否为项目规则目录
                project_rules_dir = os.path.join(self.root_dir, ".jarvis", "rules")
                is_project = rules_dir == project_rules_dir

                # 根据实际来源分配前缀
                global_rules_dir = os.path.join(get_data_dir(), "rules")

                if is_central:
                    prefix = "central:"
                elif is_project:
                    prefix = "project:"
                elif rules_dir == global_rules_dir:
                    # 全局规则目录
                    prefix = "global:"
                else:
                    # 配置的规则目录
                    prefix = "config0:"

                try:
                    for root, dirs, files in os.walk(rules_dir):
                        for filename in files:
                            if filename.endswith(".md"):
                                file_path = os.path.join(root, filename)
                                if os.path.isfile(file_path):
                                    # 计算相对于规则目录的路径作为规则名称
                                    rel_path = os.path.relpath(file_path, rules_dir)
                                    # 规则名称带来源前缀（如 project:deployment/version_release.md）
                                    prefixed_name = prefix + rel_path
                                    result["files"].append(prefixed_name)
                except Exception:
                    continue

        # 收集 rules.yaml 文件中的规则
        for desc, yaml_path in self._get_all_rules_yaml_files():
            if os.path.exists(yaml_path) and os.path.isfile(yaml_path):
                # 根据描述确定前缀
                if desc == "中心库":
                    prefix = "central_yaml:"
                elif desc == "项目":
                    prefix = "project_yaml:"
                elif desc == "全局":
                    prefix = "global_yaml:"
                else:
                    continue

                try:
                    with open(yaml_path, "r", encoding="utf-8", errors="replace") as f:
                        rules = yaml.safe_load(f) or {}
                        if isinstance(rules, dict):
                            for rule_name in rules.keys():
                                prefixed_name = prefix + rule_name
                                result["yaml"].append(prefixed_name)
                except Exception:
                    continue

        return result

    def load_all_rules(self, rule_names: Optional[str] = None) -> Tuple[str, Set[str]]:
        """加载所有规则并合并

        参数:
            rule_names: 规则名称列表（逗号分隔）

        返回:
            (merged_rules, loaded_rule_names): 合并后的规则字符串和已加载的规则名称列表
        """
        combined_parts: List[str] = []
        loaded_rule_names: Set[str] = set()

        global_rules = self.read_global_rules()
        project_rules = self.read_project_rule()

        if global_rules:
            combined_parts.append(global_rules)
            loaded_rule_names.add("global_rule")
        if project_rules:
            combined_parts.append(project_rules)
            loaded_rule_names.add("project_rule")

        # 加载 builtin_rules.md 内置规则索引
        builtin_rules_index = self._get_builtin_rules_index()
        if builtin_rules_index:
            combined_parts.append(builtin_rules_index)
            loaded_rule_names.add("builtin_rules_index")

        # 如果指定了 rule_names，从 rules.yaml 文件中读取并添加多个规则
        if rule_names:
            rule_list = [name.strip() for name in rule_names.split(",") if name.strip()]
            for rule_name in rule_list:
                named_rule = self.get_named_rule(rule_name)
                if named_rule:
                    combined_parts.append(named_rule)
                    loaded_rule_names.add(rule_name)

        if combined_parts:
            merged_rules = "\n\n".join(combined_parts)
            return merged_rules, loaded_rule_names
        return "", set()



# -*- coding: utf-8 -*-
"""TypeScript language symbol extractor."""

from typing import Any
from typing import List
from typing import Optional

from jarvis.jarvis_agent.file_context_handler import register_language_extractor


def create_typescript_extractor() -> Optional[Any]:
    """Create TypeScript symbol extractor using tree-sitter."""
    try:
        import tree_sitter_typescript
        from tree_sitter import Language
        from tree_sitter import Parser

        from jarvis.jarvis_code_agent.code_analyzer.symbol_extractor import Symbol

        # tree-sitter-typescript 使用 language_typescript() 和 language_tsx()
        TS_LANGUAGE = tree_sitter_typescript.language_typescript()
        TS_SYMBOL_QUERY = """
        (function_declaration
          name: (identifier) @function.name)
        
        (method_definition
          name: (property_identifier) @method.name)
        
        (class_declaration
          name: (type_identifier) @class.name)
        
        (interface_declaration
          name: (type_identifier) @interface.name)
        
        (variable_declaration
          (variable_declarator
            name: (identifier) @variable.name))
        """

        class TSSymbolExtractor:
            def __init__(self) -> None:
                # 如果传入的是 PyCapsule，需要转换为 Language 对象
                if not isinstance(TS_LANGUAGE, Language):
                    self.language = Language(TS_LANGUAGE)
                else:
                    self.language = TS_LANGUAGE
                self.parser = Parser()
                # 使用 language 属性而不是 set_language 方法
                self.parser.language = self.language
                self.symbol_query = TS_SYMBOL_QUERY

            def extract_symbols(self, file_path: str, content: str) -> List[Any]:
                try:
                    tree = self.parser.parse(bytes(content, "utf8"))
                    query = self.language.query(self.symbol_query)
                    captures = query.captures(tree.root_node)  # type: ignore[attr-defined]

                    symbols = []
                    for node, name in captures:
                        kind_map = {
                            "function.name": "function",
                            "method.name": "method",
                            "class.name": "class",
                            "interface.name": "interface",
                            "variable.name": "variable",
                        }
                        symbol_kind = kind_map.get(name)
                        if symbol_kind:
                            symbols.append(
                                Symbol(
                                    name=node.text.decode("utf8"),
                                    kind=symbol_kind,
                                    file_path=file_path,
                                    line_start=node.start_point[0] + 1,
                                    line_end=node.end_point[0] + 1,
                                )
                            )
                    return symbols
                except Exception:
                    return []

        return TSSymbolExtractor()
    except (ImportError, Exception):
        return None


def register_typescript_extractor() -> None:
    """Register TypeScript extractor for .ts and .tsx files."""
    register_language_extractor([".ts", ".tsx"], create_typescript_extractor)



# -*- coding: utf-8 -*-
"""Jarvis Agent 内置规则模块

提供一些优秀的开发实践规则，如 TDD、代码审查、重构等。
支持使用 jinja2 模板语法在规则文件中使用内置变量。
此模块可供所有 agent 使用。
"""

from pathlib import Path

from jarvis.jarvis_utils.template_utils import render_rule_template


# 内置规则字典：规则名称 -> 规则内容
# 此字典将在模块加载时通过 _load_builtin_rules() 函数动态填充
BUILTIN_RULES: dict[str, str] = {}


def _load_rules_from_directory(directory: Path) -> None:
    """从指定目录加载规则文件，支持jinja2模板渲染

    参数:
        directory: 规则文件所在目录
    """
    if not directory.exists():
        return

    for rule_file in directory.rglob("*.md"):
        rule_name = rule_file.stem  # 去掉 .md 后缀
        try:
            with open(rule_file, "r", encoding="utf-8") as f:
                rule_content = f.read().strip()
                # 使用通用的渲染函数
                rendered_content = render_rule_template(
                    rule_content, rule_file.parent.as_posix()
                )
                if rendered_content:
                    BUILTIN_RULES[rule_name] = rendered_content
        except Exception:
            # 忽略加载失败的文件，不影响其他规则
            continue


def _load_builtin_rules() -> None:
    """加载所有内置规则"""
    # 获取当前文件所在的项目根目录
    # 从 src/jarvis/jarvis_agent/builtin_rules.py 定位到项目根
    project_root = Path(__file__).parent.parent.parent.parent
    builtin_dir = project_root / "builtin"

    # 加载通用规则（rules 目录）
    _load_rules_from_directory(builtin_dir / "rules")

    # 加载测试规则（testing 目录）
    _load_rules_from_directory(builtin_dir / "rules" / "testing")


# 在模块加载时自动加载所有规则
_load_builtin_rules()


def get_builtin_rule(rule_name: str) -> str | None:
    """获取内置规则

    参数:
        rule_name: 规则名称（不区分大小写）

    返回:
        str: 规则内容，如果未找到则返回 None
    """
    return BUILTIN_RULES.get(rule_name.lower())


def list_builtin_rules() -> list[str]:
    """列出所有可用的内置规则名称

    返回:
        list[str]: 规则名称列表
    """
    return list(BUILTIN_RULES.keys())



# -*- coding: utf-8 -*-
"""LLM 模块规划 Agent 的项目结构应用模块。"""

import re
from pathlib import Path
from typing import Any
from typing import Dict
from typing import List
from typing import Set
from typing import Union

from jarvis.jarvis_c2rust.llm_module_agent_utils import parse_project_json_entries


def ensure_pub_mod_declarations(existing_text: str, child_mods: List[str]) -> str:
    """
    在给定文本中确保存在并升级子模块声明为 `pub mod <name>;`：
    - 解析已有的 `mod`/`pub mod`/`pub(...) mod` 声明；
    - 已存在但非 pub 的同名声明就地升级为 `pub mod`，保留原行的缩进；
    - 不存在的模块名则在末尾追加一行 `pub mod <name>;`；
    - 返回更新后的完整文本（保留结尾换行）。
    """
    try:
        lines = (existing_text or "").splitlines()
    except Exception:
        lines = []
    mod_decl_pattern = re.compile(
        r"^\s*(pub(?:\s*\([^)]+\))?\s+)?mod\s+([A-Za-z_][A-Za-z0-9_]*)\s*;\s*$"
    )
    name_to_indices: Dict[str, List[int]] = {}
    name_has_pub: Set[str] = set()
    for i, ln in enumerate(lines):
        m = mod_decl_pattern.match(ln.strip())
        if not m:
            continue
        mod_name = m.group(2)
        name_to_indices.setdefault(mod_name, []).append(i)
        if m.group(1):
            name_has_pub.add(mod_name)
    for mod_name in sorted(set(child_mods or [])):
        if mod_name in name_to_indices:
            if mod_name not in name_has_pub:
                for idx in name_to_indices[mod_name]:
                    ws_match = re.match(r"^(\s*)", lines[idx])
                    leading_ws = ws_match.group(1) if ws_match else ""
                    lines[idx] = f"{leading_ws}pub mod {mod_name};"
        else:
            lines.append(f"pub mod {mod_name};")
    return "\n".join(lines).rstrip() + ("\n" if lines else "")


def apply_entries_with_mods(entries: List[Any], base_path: Path) -> None:
    """
    根据解析出的 entries 创建目录与文件结构（不在此阶段写入/更新任何 Rust 源文件内容）：
    - 对于目录项：创建目录，并递归创建其子项；
    - 对于文件项：若不存在则创建空文件；
    约束与约定：
    - crate 根的 src 目录：不生成 src/mod.rs，也不写入 src/lib.rs 的模块声明；
    - 非 src 目录：不创建或更新 mod.rs；如需创建 mod.rs，请在 YAML 中显式列出；
    - 模块声明的补齐将在后续 CodeAgent 阶段完成（扫描目录结构并最小化补齐 pub mod 声明）。
    """

    def apply_item(item: Any, dir_path: Path) -> None:
        if isinstance(item, str):
            # 文件
            file_path = dir_path / item
            file_path.parent.mkdir(parents=True, exist_ok=True)
            if not file_path.exists():
                try:
                    file_path.touch(exist_ok=True)
                except Exception:
                    pass
            return

        if isinstance(item, dict) and len(item) == 1:
            dir_name, children = next(iter(item.items()))
            name = str(dir_name).rstrip("/").strip()
            new_dir = dir_path / name
            new_dir.mkdir(parents=True, exist_ok=True)

            child_mods: List[str] = []
            # 是否为 crate 根下的 src 目录
            is_src_root_dir = new_dir == base_path / "src"

            # 先创建子项
            for child in children or []:
                if isinstance(child, str):
                    apply_item(child, new_dir)
                    # 收集 .rs 文件作为子模块
                    if child.endswith(".rs") and child != "mod.rs":
                        stem = Path(child).stem
                        # 在 src 根目录下，忽略 lib.rs 与 main.rs 的自引用
                        if is_src_root_dir and stem in ("lib", "main"):
                            pass
                        else:
                            child_mods.append(stem)
                    if child == "mod.rs":
                        pass
                elif isinstance(child, dict):
                    # 子目录
                    sub_name = list(child.keys())[0]
                    sub_mod_name = str(sub_name).rstrip("/").strip()
                    child_mods.append(sub_mod_name)
                    apply_item(child, new_dir)

            # 对 crate 根的 src 目录，使用 lib.rs 聚合子模块，不创建/更新 src/mod.rs
            if is_src_root_dir:
                # 不在 src 根目录写入任何文件内容；仅由子项创建对应空文件（如有）
                return

            # 非 src 目录：
            # 为避免覆盖现有实现，当前阶段不创建或更新 mod.rs 内容。
            # 如需创建 mod.rs，应在 JSON 中显式指定为文件项；
            # 如需补齐模块声明，将由后续的 CodeAgent 阶段根据目录结构自动补齐。
            return

    for entry in entries:
        apply_item(entry, base_path)


def ensure_cargo_toml(base_dir: Path, package_name: str) -> None:
    """
    确保在 base_dir 下存在合理的 Cargo.toml：
    - 如果不存在，则创建最小可用的 Cargo.toml，并设置 package.name = package_name
    - 如果已存在，则不覆盖现有内容（避免误改）
    """
    cargo_path = base_dir / "Cargo.toml"
    if cargo_path.exists():
        return
    try:
        cargo_path.touch(exist_ok=True)
    except (OSError, PermissionError):
        # 如果无法创建文件，记录错误但不中断流程
        # 后续 CodeAgent 可能会处理 Cargo.toml 的创建
        pass


def apply_project_structure_from_json(
    json_text: str, project_root: Union[Path, str] = "."
) -> None:
    """
    基于 Agent 返回的 <PROJECT> 中的目录结构 JSON，创建实际目录与文件（不在此阶段写入或更新任何 Rust 源文件内容）。
    - project_root: 目标应用路径；当为 "."（默认）时，将使用"父目录/当前目录名_rs"作为crate根目录
    注意：模块声明（mod/pub mod）补齐将在后续的 CodeAgent 步骤中完成。按新策略不再创建或更新 workspace（构建直接在 crate 目录内进行）。
    """
    entries, parse_error = parse_project_json_entries(json_text)
    if parse_error:
        raise ValueError(f"JSON解析失败: {parse_error}")
    if not entries:
        # 严格模式：解析失败直接报错并退出，由上层 CLI 捕获打印错误
        raise ValueError("[c2rust-llm-planner] 从LLM输出解析目录结构失败。正在中止。")
    requested_root = Path(project_root).resolve()
    try:
        cwd = Path(".").resolve()
        if requested_root == cwd:
            # 默认crate不能设置为 .，设置为 父目录/当前目录名_rs（与当前目录同级）
            base_dir = cwd.parent / f"{cwd.name}_rs"
        else:
            base_dir = requested_root
    except Exception:
        base_dir = requested_root
    base_dir.mkdir(parents=True, exist_ok=True)
    # crate name 与目录名保持一致（用于 Cargo 包名，允许连字符）
    crate_pkg_name = base_dir.name
    apply_entries_with_mods(entries, base_dir)
    # 确保 Cargo.toml 存在并设置包名
    ensure_cargo_toml(base_dir, crate_pkg_name)

    # 已弃用：不再将 crate 添加到 workspace（按新策略去除 workspace）
    # 构建与工具运行将直接在 crate 目录内进行



# -*- coding: utf-8 -*-
"""
FastAPI Web 应用

提供配置表单的 Web 服务
"""

import json
from pathlib import Path
from typing import Any, Dict, Optional

import yaml

from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import HTMLResponse
from pydantic import BaseModel

from .schema_parser import SchemaParser

from jarvis.jarvis_utils.output import PrettyOutput


# 请求模型
class SaveConfigRequest(BaseModel):
    """保存配置的请求模型"""

    config: Dict[str, Any]


# 存储全局状态（在实际应用中应该使用更好的状态管理）
_schema_parser: Optional[SchemaParser] = None
_output_path: Optional[Path] = None
_existing_config: Dict[str, Any] = {}


def create_app(schema_path: Path, output_path: Path) -> FastAPI:
    """创建 FastAPI 应用

    Args:
        schema_path: JSON Schema 文件路径
        output_path: 输出配置文件路径

    Returns:
        FastAPI 应用实例
    """
    global _schema_parser, _output_path, _existing_config

    # 初始化 schema 解析器
    _schema_parser = SchemaParser(schema_path)
    _output_path = output_path

    # 加载现有配置文件（如果存在）
    _existing_config = {}
    if output_path.exists():
        try:
            with open(output_path, "r", encoding="utf-8") as f:
                if output_path.suffix in (".yaml", ".yml"):
                    _existing_config = yaml.safe_load(f) or {}
                else:
                    _existing_config = json.load(f)
        except Exception:
            # 如果加载失败，使用空配置
            _existing_config = {}

    # 创建 FastAPI 应用
    app = FastAPI(
        title="Jarvis 配置工具",
        description="基于 JSON Schema 的动态配置表单",
        version="1.0.0",
    )

    # 启用 CORS
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    @app.get("/", response_class=HTMLResponse)
    async def index() -> str:
        """返回 HTML 页面"""
        # TODO: 在任务4中实现完整的 Zen-iOS Hybrid 风格前端
        # 这里先返回一个基础框架
        return get_html_template()

    @app.get("/api/schema")
    async def get_schema() -> Dict[str, Any]:
        """获取 Schema 数据

        Returns:
            Schema 对象，包含属性、类型、约束等信息
        """
        if _schema_parser is None:
            raise HTTPException(status_code=500, detail="Schema parser not initialized")

        properties = _schema_parser.get_properties()

        # 为每个属性添加额外的元数据
        for prop_name in properties:
            schema_default = _schema_parser.get_default_value(prop_name)
            # 如果现有配置中有该属性的值，则用该值覆盖默认值
            if prop_name in _existing_config:
                schema_default = _existing_config[prop_name]

            properties[prop_name]["_meta"] = {
                "default": schema_default,
                "enum": _schema_parser.get_enum(prop_name),
                "description": _schema_parser.get_description_for_property(prop_name),
                "required": prop_name in _schema_parser.get_required(),
            }

        return {
            "title": _schema_parser.get_title(),
            "description": _schema_parser.get_description(),
            "properties": properties,
            "required": _schema_parser.get_required(),
        }

    @app.post("/api/save")
    async def save_config(request: SaveConfigRequest) -> Dict[str, Any]:
        """保存配置

        Args:
            request: 配置数据请求

        Returns:
            保存结果
        """
        if _schema_parser is None:
            raise HTTPException(status_code=500, detail="Schema parser not initialized")

        if _output_path is None:
            raise HTTPException(status_code=500, detail="Output path not set")

        # 调试：打印保存信息
        PrettyOutput.auto_print(
            f"🔍 [DEBUG] 准备保存配置到: {_output_path}", timestamp=False
        )
        PrettyOutput.auto_print(
            f"🔍 [DEBUG] 文件路径类型: {type(_output_path)}", timestamp=False
        )
        PrettyOutput.auto_print(
            f"🔍 [DEBUG] 文件路径存在: {_output_path.exists()}", timestamp=False
        )

        # 清理配置中的 null 值（递归移除）
        def clean_null_values(obj: Any) -> Any:
            if isinstance(obj, dict):
                return {
                    k: clean_null_values(v) for k, v in obj.items() if v is not None
                }
            elif isinstance(obj, list):
                return [clean_null_values(item) for item in obj if item is not None]
            return obj

        cleaned_config = clean_null_values(request.config)
        PrettyOutput.auto_print(
            f"🔍 [DEBUG] 清理 null 值后的配置键数: {len(cleaned_config)}",
            timestamp=False,
        )

        # 验证配置
        errors = _schema_parser.validate_config(cleaned_config)
        PrettyOutput.auto_print(
            f"🔍 [DEBUG] 验证结果: {len(errors) if errors else 0} 个错误",
            timestamp=False,
        )
        if errors:
            for error in errors:
                PrettyOutput.auto_print(
                    f"🔍 [DEBUG] 验证错误: path={error.path}, message={error.message}",
                    timestamp=False,
                )
            return {
                "success": False,
                "errors": [
                    {"path": error.path, "message": error.message} for error in errors
                ],
            }

        # 保存配置文件
        try:
            # 确保输出目录存在
            _output_path.parent.mkdir(parents=True, exist_ok=True)

            # 根据文件后缀决定格式
            PrettyOutput.auto_print(
                f"🔍 [DEBUG] 开始写入文件，后缀: {_output_path.suffix}", timestamp=False
            )
            if _output_path.suffix in (".yaml", ".yml"):
                with open(_output_path, "w", encoding="utf-8") as f:
                    yaml.dump(
                        cleaned_config,
                        f,
                        allow_unicode=True,
                        default_flow_style=False,
                        sort_keys=False,
                    )
                PrettyOutput.auto_print("🔍 [DEBUG] YAML 写入完成", timestamp=False)
            else:
                with open(_output_path, "w", encoding="utf-8") as f:
                    json.dump(cleaned_config, f, indent=2, ensure_ascii=False)
                PrettyOutput.auto_print("🔍 [DEBUG] JSON 写入完成", timestamp=False)

            # 验证写入结果
            import os

            PrettyOutput.auto_print(
                f"🔍 [DEBUG] 写入后文件大小: {os.path.getsize(_output_path)} 字节",
                timestamp=False,
            )
            PrettyOutput.auto_print(
                f"🔍 [DEBUG] 写入后文件修改时间: {os.path.getmtime(_output_path)}",
                timestamp=False,
            )

            return {
                "success": True,
                "message": f"配置已保存到 {_output_path}",
                "path": str(_output_path),
            }
        except Exception as e:
            raise HTTPException(status_code=500, detail=f"保存配置失败: {str(e)}")

    @app.get("/api/health")
    async def health_check() -> Dict[str, str]:
        """健康检查"""
        return {"status": "ok"}

    @app.post("/api/shutdown")
    async def shutdown() -> Dict[str, str]:
        """关闭服务器"""
        import threading
        import os
        import signal

        def delayed_shutdown():
            """延迟关闭，确保响应先发送"""
            import time

            time.sleep(0.5)
            os.kill(os.getpid(), signal.SIGTERM)

        threading.Thread(target=delayed_shutdown, daemon=True).start()
        return {"status": "shutting_down"}

    return app


def get_html_template() -> str:
    """获取 HTML 模板

    Returns:
        HTML 字符串
    """
    return r"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Jarvis 配置工具</title>
    <script>
        function expandAll() {
            const sections = document.querySelectorAll('.form-section');
            
            // 对于每个折叠的区域，模拟点击其折叠按钮以确保状态一致
            for (let i = 0; i < sections.length; i++) {
                const toggleBtn = sections[i].querySelector('.collapse-toggle');
                if (toggleBtn && sections[i].classList.contains('collapsed')) {
                    // 模拟点击操作，确保调用toggleCollapse函数
                    toggleBtn.click();
                }
            }
        }
        
        function collapseAll() {
            const sections = document.querySelectorAll('.form-section');
            
            // 对于每个展开的区域，模拟点击其折叠按钮以确保状态一致
            for (let i = 0; i < sections.length; i++) {
                const toggleBtn = sections[i].querySelector('.collapse-toggle');
                if (toggleBtn && !sections[i].classList.contains('collapsed')) {
                    // 模拟点击操作，确保调用toggleCollapse函数
                    toggleBtn.click();
                }
            }
        }
    </script>
    <style>
        :root {
            --bg-primary: #F2F2F7;
            --bg-glass: rgba(255, 255, 255, 0.5);
            --bg-input: rgba(243, 244, 246, 0.5);
            --bg-input-focus: rgba(243, 244, 246, 0.7);
            --text-primary: #1C1C1E;
            --text-secondary: #6B7280;
            --text-label: #3C3C43;
            --border-inner: rgba(255, 255, 255, 0.6);
            --border-outer: rgba(0, 0, 0, 0.08);
            --button-primary: #1C1C1E;
            --button-primary-hover: #2C2C2E;
            --error: #DC2626;
            --error-bg: #FEE2E2;
            --shadow-float: 0 24px 48px -12px rgba(0, 0, 0, 0.08);
            --shadow-inset: inset 0 2px 4px rgba(0, 0, 0, 0.06);
        }

        * {
            margin: 0;
            padding: 0;
            box-sizing: border-box;
        }

        body {
            font-family: -apple-system, BlinkMacSystemFont, "SF Pro Display", "Inter", sans-serif;
            background: var(--bg-primary);
            color: var(--text-primary);
            line-height: 1.4;
            padding: 12px;
            min-height: 100vh;
        }

        .main-container {
            max-width: 1400px;
            margin: 0 auto;
        }

        #form-fields {
            display: grid;
            grid-template-columns: 1fr;
            gap: 10px;
        }

        /* 宽屏多列布局 */
        @media (min-width: 1024px) {
            #form-fields {
                grid-template-columns: repeat(2, 1fr);
            }

            /* 复杂字段（包含嵌套对象或数组）占满整行 */
            .form-section:has(.nested-object),
            .form-section:has(.dict-container),
            .form-section:has(.array-item) {
                grid-column: 1 / -1;
            }
        }

        /* 超宽屏三列布局 */
        @media (min-width: 1600px) {
            #form-fields {
                grid-template-columns: repeat(3, 1fr);
            }
        }

        .glass-card {
            background: var(--bg-glass);
            backdrop-filter: blur(50px);
            -webkit-backdrop-filter: blur(50px);
            border-radius: 20px;
            padding: 20px;
            border: 1px solid var(--border-inner);
            box-shadow: 0 0 0 1px var(--border-outer), var(--shadow-float);
            margin-bottom: 12px;
        }

        h1 {
            font-weight: 800;
            letter-spacing: -0.02em;
            font-size: 22px;
            margin-bottom: 4px;
            color: var(--text-primary);
        }

        .description {
            color: var(--text-secondary);
            font-size: 13px;
            line-height: 1.4;
            margin-bottom: 16px;
        }

        .loading, .error, .success {
            text-align: center;
            padding: 24px 16px;
            border-radius: 12px;
            background: var(--bg-glass);
            backdrop-filter: blur(50px);
        }

        .loading {
            color: var(--text-secondary);
            font-size: 16px;
        }

        .error {
            background: var(--error-bg);
            color: var(--error);
            border: 1px solid rgba(220, 38, 38, 0.2);
        }

        .success {
            background: rgba(16, 185, 129, 0.1);
            color: #059669;
            border: 1px solid rgba(16, 185, 129, 0.2);
        }

        #form-container {
            display: none;
        }

        .form-section {
            background: var(--bg-glass);
            backdrop-filter: blur(40px);
            border-radius: 12px;
            padding: 0;
            border: 1px solid var(--border-inner);
            box-shadow: 0 0 0 1px var(--border-outer), var(--shadow-float);
            /* Grid 布局时不再需要 margin-bottom，由 gap 控制 */
        }
        
        .form-section-header {
            padding: 8px;
            border-bottom: 1px solid var(--border-inner);
            cursor: pointer;
            display: flex;
            align-items: center;
        }
        
        .collapse-toggle {
            background: none;
            border: none;
            font-size: 14px;
            cursor: pointer;
            color: var(--text-secondary);
            padding: 1px 4px;
            border-radius: 3px;
            margin-left: auto;
        }
        
        .form-section-header:hover {
            background: var(--bg-input-focus);
            border-radius: 12px 12px 0 0;
        }
        
        .form-section-content {
            padding: 8px;
            display: block;
        }
        
        .form-section.collapsed .form-section-content {
            display: none;
        }
        
        /* 添加CSS优化以提高性能 */
        .form-section {
            will-change: transform;
        }
        
        .form-section-header {
            cursor: pointer;
            user-select: none; /* 防止误选文本 */
        }
        
        .collapse-toggle {
            background: none;
            border: none;
            font-size: 14px;
            cursor: pointer;
            color: var(--text-secondary);
            padding: 1px 4px;
            border-radius: 3px;
        }
        
        .collapse-toggle:hover {
            background: var(--bg-input);
            color: var(--text-primary);
        }

        .form-group {
            margin-bottom: 6px;
        }

        .form-group:last-child {
            margin-bottom: 0;
        }

        label {
            display: block;
            font-weight: 700;
            font-size: 11px;
            text-transform: uppercase;
            letter-spacing: 0.1em;
            color: var(--text-label);
            margin-bottom: 4px;
        }

        .required-mark {
            color: var(--error);
            margin-left: 2px;
        }

        .field-description {
            display: inline-block;
            color: var(--text-secondary);
            font-size: 11px;
            margin-left: 6px;
            line-height: 1.3;
            vertical-align: middle;
        }

        .field-description.deprecated {
            color: #9CA3AF;
            font-style: italic;
            background: linear-gradient(135deg, rgba(156, 163, 175, 0.1), rgba(156, 163, 175, 0.05));
            padding: 4px 8px;
            border-radius: 4px;
            border-left: 3px solid #F59E0B;
        }

        .field-description.deprecated::before {
            content: "⚠️ ";
        }

        input[type="text"],
        input[type="number"],
        input[type="email"],
        input[type="url"],
        input[type="tel"],
        textarea,
        select {
            width: 100%;
            padding: 4px 8px;
            border: 1px solid var(--border-inner);
            border-radius: 6px;
            background: var(--bg-input);
            font-size: 13px;
            color: var(--text-primary);
            outline: none;
            transition: all 0.2s ease;
            box-shadow: var(--shadow-inset);
            font-family: -apple-system, BlinkMacSystemFont, "SF Pro Display", "Inter", sans-serif;
        }

        input:focus,
        textarea:focus,
        select:focus {
            background: var(--bg-input-focus);
            box-shadow: var(--shadow-inset), 0 0 0 3px rgba(0, 122, 255, 0.1);
        }

        input::placeholder,
        textarea::placeholder {
            color: var(--text-secondary);
        }

        textarea {
            min-height: 60px;
            resize: vertical;
        }

        .checkbox-group,
        .radio-group {
            display: flex;
            flex-direction: column;
            gap: 6px;
        }

        .checkbox-item,
        .radio-item {
            display: flex;
            align-items: center;
            padding: 8px 12px;
            background: var(--bg-input);
            border: 1px solid var(--border-inner);
            border-radius: 8px;
            cursor: pointer;
            transition: all 0.2s ease;
            box-shadow: var(--shadow-inset);
        }

        .checkbox-item:hover,
        .radio-item:hover {
            background: var(--bg-input-focus);
        }

        .checkbox-item:active,
        .radio-item:active {
            transform: scale(0.98);
        }

        .checkbox-item input,
        .radio-item input {
            width: 16px;
            height: 16px;
            margin-right: 8px;
            cursor: pointer;
            accent-color: var(--button-primary);
        }

        .switch-container {
            display: flex;
            align-items: center;
            justify-content: space-between;
            padding: 6px 12px;
            background: var(--bg-input);
            border: 1px solid var(--border-inner);
            border-radius: 8px;
            box-shadow: var(--shadow-inset);
        }

        .switch {
            position: relative;
            width: 40px;
            height: 24px;
            flex-shrink: 0;
        }

        .switch input {
            opacity: 0;
            width: 0;
            height: 0;
        }

        .switch-slider {
            position: absolute;
            cursor: pointer;
            top: 0;
            left: 0;
            right: 0;
            bottom: 0;
            background-color: #D1D1D6;
            transition: 0.3s;
            border-radius: 12px;
        }

        .switch-slider:before {
            position: absolute;
            content: "";
            height: 20px;
            width: 20px;
            left: 2px;
            bottom: 2px;
            background-color: white;
            transition: 0.3s;
            border-radius: 50%;
            box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
        }

        .switch input:checked + .switch-slider {
            background-color: #34C759;
        }

        .switch input:checked + .switch-slider:before {
            transform: translateX(16px);
        }

        .switch:active .switch-slider:before {
            width: 24px;
        }

        .nested-object {
            background: rgba(243, 244, 246, 0.3);
            border-radius: 10px;
            padding: 12px;
            margin-top: 6px;
            border: 1px solid var(--border-inner);
        }

        .dict-container {
            background: var(--bg-input);
            border: 1px solid var(--border-inner);
            border-radius: 10px;
            padding: 10px;
        }

        .dict-item {
            background: var(--bg-glass);
            border: 1px solid var(--border-inner);
            border-radius: 8px;
            padding: 10px;
            margin-bottom: 8px;
            box-shadow: var(--shadow-inset);
        }

        .dict-item:last-child {
            margin-bottom: 0;
        }

        .dict-key-input {
            width: 100%;
            padding: 6px 10px;
            border: 1px solid var(--border-inner);
            border-radius: 6px;
            background: var(--bg-primary);
            font-size: 13px;
            color: var(--text-primary);
            outline: none;
            margin-bottom: 8px;
            box-shadow: var(--shadow-inset);
        }

        .dict-key-input:focus {
            background: rgba(243, 244, 246, 0.5);
            box-shadow: var(--shadow-inset), 0 0 0 3px rgba(0, 122, 255, 0.1);
        }

        .dict-controls {
            display: flex;
            gap: 8px;
            margin-top: 8px;
        }

        .dict-item-header {
            display: flex;
            justify-content: space-between;
            align-items: center;
            margin-bottom: 6px;
        }

        .dict-item-remove {
            margin-left: auto;
            flex-shrink: 0;
        }

        .array-item {
            background: var(--bg-input);
            border: 1px solid var(--border-inner);
            border-radius: 10px;
            padding: 10px;
            margin-bottom: 8px;
            box-shadow: var(--shadow-inset);
        }

        .array-item:last-child {
            margin-bottom: 0;
        }

        .array-controls {
            display: flex;
            gap: 8px;
            margin-top: 8px;
        }

        .btn {
            padding: 8px 16px;
            border: none;
            border-radius: 8px;
            font-weight: 600;
            font-size: 13px;
            cursor: pointer;
            transition: all 0.2s ease;
            font-family: -apple-system, BlinkMacSystemFont, "SF Pro Display", "Inter", sans-serif;
        }

        .btn:active {
            transform: scale(0.98);
        }

        .btn-primary {
            background: var(--button-primary);
            color: white;
        }

        .btn-primary:hover {
            background: var(--button-primary-hover);
        }

        .btn-secondary {
            background: white;
            color: var(--text-primary);
            box-shadow: 0 2px 8px rgba(0, 0, 0, 0.1);
        }

        .btn-secondary:hover {
            box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);
        }

        .btn-danger {
            background: rgba(220, 38, 38, 0.1);
            color: var(--error);
            border: 1px solid rgba(220, 38, 38, 0.2);
        }

        .btn-danger:hover {
            background: rgba(220, 38, 38, 0.15);
        }

        .btn-sm {
            padding: 5px 12px;
            font-size: 12px;
            border-radius: 6px;
        }

        .btn-icon {
            width: 26px;
            height: 26px;
            padding: 0;
            display: flex;
            align-items: center;
            justify-content: center;
            border-radius: 6px;
        }

        .form-actions {
            margin-top: 16px;
            display: flex;
            gap: 10px;
        }

        .field-error {
            color: var(--error);
            font-size: 11px;
            margin-top: 4px;
            display: none;
        }

        .has-error input,
        .has-error select,
        .has-error textarea {
            border-color: var(--error);
        }

        .has-error .field-error {
            display: block;
        }

        @media (max-width: 768px) {
            body {
                padding: 16px;
            }

            .glass-card {
                border-radius: 32px;
                padding: 32px 24px;
            }

            h1 {
                font-size: 28px;
            }

            .form-section {
                border-radius: 24px;
                padding: 24px 20px;
            }

            .form-actions {
                flex-direction: column;
            }

            .btn {
                width: 100%;
            }
        }
    </style>
</head>
<body>
    <div class="main-container">
        <div class="glass-card">
            <h1 id="title">Loading...</h1>
            <p id="description" class="description"></p>
            
            <div id="loading" class="loading">
                <div style="margin-bottom: 16px;">
                    <svg xmlns="http://www.w3.org/2000/svg" width="48" height="48" viewBox="0 0 24 24" fill="none" stroke="#6B7280" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round">
                        <circle cx="12" cy="12" r="3"></circle>
                        <path d="M19.4 15a1.65 1.65 0 0 0 .33 1.82l.06.06a2 2 0 0 1 0 2.83 2 2 0 0 1-2.83 0l-.06-.06a1.65 1.65 0 0 0-1.82-.33 1.65 1.65 0 0 0-1 1.51V21a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-.09A1.65 1.65 0 0 0 9 19.4a1.65 1.65 0 0 0-1.82.33l-.06.06a2 2 0 0 1-2.83 0 2 2 0 0 1 0-2.83l.06-.06a1.65 1.65 0 0 0 .33-1.82 1.65 1.65 0 0 0-1.51-1H3a2 2 0 0 1-2-2 2 2 0 0 1 2-2h.09A1.65 1.65 0 0 0 4.6 9a1.65 1.65 0 0 0-.33-1.82l-.06-.06a2 2 0 0 1 0-2.83 2 2 0 0 1 2.83 0l.06.06a1.65 1.65 0 0 0 1.82.33H9a1.65 1.65 0 0 0 1-1.51V3a2 2 0 0 1 2-2 2 2 0 0 1 2 2v.09a1.65 1.65 0 0 0 1 1.51 1.65 1.65 0 0 0 1.82-.33l.06-.06a2 2 0 0 1 2.83 0 2 2 0 0 1 0 2.83l-.06.06a1.65 1.65 0 0 0-.33 1.82V9a1.65 1.65 0 0 0 1.51 1H21a2 2 0 0 1 2 2 2 2 0 0 1-2 2h-.09a1.65 1.65 0 0 0-1.51 1z"></path>
                    </svg>
                </div>
                加载配置表单中...
            </div>
            
            <div id="error" class="error" style="display: none;"></div>
            <div id="success" class="success" style="display: none;"></div>
            
            <div id="form-container">
                <form id="config-form" novalidate>
                    <div style="display: flex; gap: 10px; margin-bottom: 16px;">
                        <button type="button" class="btn btn-secondary" onclick="expandAll()">展开所有</button>
                        <button type="button" class="btn btn-secondary" onclick="collapseAll()">折叠所有</button>
                    </div>
                    <div id="form-fields"></div>
                    <div class="form-actions">
                        <button type="submit" class="btn btn-primary">保存配置</button>
                    </div>
                </form>
            </div>
        </div>
    </div>

    <script>
        let schemaData = null;
        let formData = {};
        let arrayCounters = {};
        let dictCounters = {};

        async function loadSchema() {
            try {
                const response = await fetch('/api/schema');
                if (!response.ok) throw new Error('加载 Schema 失败');
                schemaData = await response.json();
                renderForm();
            } catch (error) {
                document.getElementById('loading').style.display = 'none';
                document.getElementById('error').style.display = 'block';
                document.getElementById('error').textContent = error.message;
            }
        }

        function renderForm() {
            document.getElementById('loading').style.display = 'none';
            document.getElementById('form-container').style.display = 'block';

            document.getElementById('title').textContent = schemaData.title || '配置';
            document.getElementById('description').textContent = schemaData.description || '';

            const fieldsContainer = document.getElementById('form-fields');
            fieldsContainer.innerHTML = '';
            
            // 使用 DocumentFragment 来批量添加元素，减少重排次数
            const fragment = document.createDocumentFragment();

            for (const [name, prop] of Object.entries(schemaData.properties)) {
                const meta = prop._meta || {};
                const isRequired = schemaData.required.includes(name);

                const section = document.createElement('div');
                section.className = 'form-section';
                section.innerHTML = createFieldHTML(name, prop, meta, isRequired, []);
                fragment.appendChild(section);
            }
            
            fieldsContainer.appendChild(fragment);

            initializeFormValues();
        }

        function createFieldHTML(name, prop, meta, isRequired, path) {
            const type = prop.type;
            const fullPath = [...path, name];
            const fullPathStr = fullPath.join('.');
            const labelText = name + (isRequired ? '<span class="required-mark">*</span>' : '');
            
            // 创建可折叠的表单区域，默认折叠
            let html = '<div class="form-section collapsed">';
            html += '<div class="form-section-header">';
            html += '<span>' + labelText + '</span>';
            
            // 如果有描述，将其添加到标签后面
            if (meta.description) {
                const isDeprecated = meta.description.includes('[已废弃');
                const descClass = isDeprecated ? 'field-description deprecated' : 'field-description';
                html += '<span class="' + descClass + '">' + escapeHtml(meta.description) + '</span>';
            }
            
            html += '<button type="button" class="collapse-toggle">▶</button>';
            html += '</div>';
            html += '<div class="form-section-content">';
            html += '<div class="form-group" data-path="' + fullPathStr + '">';

            if (prop.enum) {
                html += createSelectHTML(fullPathStr, prop.enum, meta.default);
            } else if (type === 'boolean') {
                html += createSwitchHTML(fullPathStr, meta.default);
            } else if (type === 'string' && prop.format === 'textarea') {
                html += createTextareaHTML(fullPathStr, prop, meta.default);
            } else if (type === 'number' || type === 'integer') {
                html += createNumberInputHTML(fullPathStr, prop, meta.default);
            } else if (type === 'array') {
                html += createArrayHTML(fullPathStr, prop, meta.default, fullPath);
            } else if (type === 'object') {
                // 判断是固定属性对象还是字典类型
                if (prop.additionalProperties && !prop.properties) {
                    html += createDictHTML(fullPathStr, prop, meta.default, fullPath);
                } else {
                    html += createObjectHTML(fullPathStr, prop, meta.default, fullPath);
                }
            } else {
                html += createTextInputHTML(fullPathStr, prop, meta.default);
            }

            html += '<div class="field-error" id="error-' + fullPathStr + '"></div>';
            html += '</div>';
            html += '</div>';
            html += '</div>';

            return html;
        }

        function createTextInputHTML(path, prop, defaultValue) {
            const placeholder = prop.examples ? prop.examples[0] : '';
            const value = defaultValue !== undefined ? defaultValue : '';
            return '<input type="text" name="' + path + '" placeholder="' + escapeHtml(placeholder) + '" value="' + escapeHtml(String(value)) + '">';
        }

        function createNumberInputHTML(path, prop, defaultValue) {
            const min = prop.minimum !== undefined ? prop.minimum : '';
            const max = prop.maximum !== undefined ? prop.maximum : '';
            const value = defaultValue !== undefined ? defaultValue : '';
            return '<input type="number" name="' + path + '" min="' + min + '" max="' + max + '" value="' + value + '">';
        }

        function createTextareaHTML(path, prop, defaultValue) {
            const minLength = prop.minLength !== undefined ? prop.minLength : '';
            const maxLength = prop.maxLength !== undefined ? prop.maxLength : '';
            const value = defaultValue !== undefined ? defaultValue : '';
            return '<textarea name="' + path + '" minlength="' + minLength + '" maxlength="' + maxLength + '" placeholder="输入内容...">' + escapeHtml(String(value)) + '</textarea>';
        }

        function createSelectHTML(path, enumValues, defaultValue) {
            let html = '<select name="' + path + '">';
            if (defaultValue === undefined) {
                html += '<option value="">请选择...</option>';
            }
            enumValues.forEach(function(value) {
                const selected = value === defaultValue ? ' selected' : '';
                html += '<option value="' + escapeHtml(String(value)) + '"' + selected + '>' + escapeHtml(String(value)) + '</option>';
            });
            html += '</select>';
            return html;
        }

        function createSwitchHTML(path, defaultValue) {
            const checked = defaultValue === true ? 'checked' : '';
            return '<div class="switch-container">\n                <span>' + (defaultValue === true ? '已启用' : '已禁用') + '</span>\n                <label class="switch">\n                    <input type="checkbox" name="' + path + '" value="true" ' + checked + '>\n                    <span class="switch-slider"></span>\n                </label>\n            </div>';
        }

        function createArrayHTML(path, prop, defaultValue, parentPath) {
            // path 已经是完整路径字符串
            // parentPath 不再使用，保留参数以保持兼容性
            const fullPathStr = path;
            
            if (!arrayCounters[fullPathStr]) {
                arrayCounters[fullPathStr] = 0;
            }

            // 创建数组容器，不具有独立折叠功能
            let html = '<div class="array-container" data-path="' + fullPathStr + '">';
            html += '<div id="array-items-' + fullPathStr + '"></div>';
            html += '<div class="array-controls">';
            html += '<button type="button" class="btn btn-secondary btn-sm" onclick="addArrayItem(\'' + fullPathStr + '\')">+ 添加项</button>';
            html += '</div></div>';
            html += '</div>';
            html += '</div>';

            setTimeout(function() {
                const itemsContainer = document.getElementById('array-items-' + fullPathStr);
                if (itemsContainer) {
                    itemsContainer.dataset.schema = JSON.stringify(prop.items || {});
                }
            }, 0);

            return html;
        }

        function addArrayItem(path) {
            const container = document.getElementById('array-items-' + path);
            if (!container) return;

            const schema = JSON.parse(container.dataset.schema || '{}');
            const index = arrayCounters[path]++;
            const itemPath = path + '[' + index + ']';

            const itemDiv = document.createElement('div');
            itemDiv.className = 'array-item';
            itemDiv.dataset.index = index;

            let fieldHTML = '';
            if (schema.type === 'object') {
                for (const propName in schema.properties || {}) {
                    const propSchema = schema.properties[propName];
                    const propMeta = propSchema._meta || {};
                    const propRequired = (schema.required || []).includes(propName);
                    fieldHTML += createFieldHTML(propName, propSchema, propMeta, propRequired, [itemPath]);
                }
            } else {
                const meta = schema._meta || {};
                fieldHTML += createFieldHTML('value', schema, meta, false, [itemPath]);
            }

            itemDiv.innerHTML = fieldHTML;
            itemDiv.innerHTML += '<button type="button" class="btn btn-danger btn-sm btn-icon" onclick="removeArrayItem(\'' + path + '\', ' + index + ')" style="margin-top: 16px;">×</button>';

            container.appendChild(itemDiv);
        }

        function removeArrayItem(path, index) {
            const container = document.getElementById('array-items-' + path);
            if (!container) return;

            const item = container.querySelector('[data-index="' + index + '"]');
            if (item) {
                item.remove();
            }
        }

        function createObjectHTML(path, prop, defaultValue, parentPath) {
            // path 已经是完整路径字符串
            // parentPath 是包含完整路径的数组，用于传递给子字段
            const fullPathStr = path;
            
            // 创建对象容器，不具有独立折叠功能
            let html = '<div class="nested-object" data-path="' + fullPathStr + '">';
            
            for (const propName in prop.properties || {}) {
                const propSchema = prop.properties[propName];
                const propMeta = propSchema._meta || {};
                const propRequired = (prop.required || []).includes(propName);
                html += createFieldHTML(propName, propSchema, propMeta, propRequired, parentPath);
            }
            
            html += '</div>';
            html += '</div>';
            html += '</div>';
            return html;
        }

        function createDictHTML(path, prop, defaultValue, parentPath) {
            // path 已经是完整路径字符串（如 "llms[glm].llm_config"）
            // parentPath 不再使用，保留参数以保持兼容性
            const fullPathStr = path;
            
            if (!dictCounters[fullPathStr]) {
                dictCounters[fullPathStr] = Object.keys(defaultValue || {}).length;
            }
            
            // 创建字典容器，不具有独立折叠功能
            let html = '<div class="dict-container" data-path="' + fullPathStr + '">';
            html += '<div id="dict-items-' + fullPathStr + '"></div>';
            html += '<div class="dict-controls">';
            html += '<button type="button" class="btn btn-secondary btn-sm" onclick="addDictItem(\'' + fullPathStr + '\')">+ 添加条目</button>';
            html += '</div></div>';
            html += '</div>';
            html += '</div>';
            
            setTimeout(function() {
                const itemsContainer = document.getElementById('dict-items-' + fullPathStr);
                if (itemsContainer) {
                    itemsContainer.dataset.schema = JSON.stringify(prop.additionalProperties || {});
                    itemsContainer.dataset.defaultValue = JSON.stringify(defaultValue || {});
                    // 初始化已有条目
                    for (const key in defaultValue || {}) {
                        addDictItem(fullPathStr, key, defaultValue[key]);
                    }
                }
            }, 0);
            
            return html;
        }

        function addDictItem(path, existingKey, existingValue) {
            const container = document.getElementById('dict-items-' + path);
            if (!container) return;

            const schema = JSON.parse(container.dataset.schema || '{}');
            const itemId = 'dict-' + Date.now() + '-' + Math.random().toString(36).substr(2, 9);

            const itemDiv = document.createElement('div');
            itemDiv.className = 'dict-item';
            itemDiv.dataset.id = itemId;

            const key = existingKey || '';
            let fieldHTML = '';

            if (existingKey) {
                // 编辑已有条目，创建键名显示和值字段
                fieldHTML = '<div class="dict-item-header">';
                fieldHTML += '<label>键名: ' + escapeHtml(String(existingKey)) + '</label>';
                fieldHTML += '<button type="button" class="btn btn-danger btn-sm btn-icon dict-item-remove" onclick="removeDictItem(\'' + path + '\', \'' + itemId + '\')">×</button>';
                fieldHTML += '</div>';

                // 根据值类型创建不同的字段
                const valuePath = [path + '[' + existingKey + ']'];
                if (schema.type === 'object' && schema.properties) {
                    // 对象类型：遍历所有属性
                    for (const propName in schema.properties || {}) {
                        const propSchema = schema.properties[propName];
                        const propDefaultValue = (existingValue || {})[propName];
                        const propMeta = {
                            default: propDefaultValue !== undefined ? propDefaultValue : propSchema.default,
                            description: propSchema.description,
                            enum: propSchema.enum
                        };
                        const propRequired = (schema.required || []).includes(propName);
                        fieldHTML += createFieldHTML(propName, propSchema, propMeta, propRequired, valuePath);
                    }
                } else {
                    // 非对象类型：直接使用路径作为字段名，不添加额外后缀
                    const fieldPath = path + '[' + existingKey + ']';
                    const defaultVal = existingValue !== undefined ? existingValue : (schema.default || '');
                    if (schema.type === 'boolean') {
                        fieldHTML += createSwitchHTML(fieldPath, defaultVal);
                    } else if (schema.type === 'number' || schema.type === 'integer') {
                        fieldHTML += createNumberInputHTML(fieldPath, schema, defaultVal);
                    } else {
                        fieldHTML += createTextInputHTML(fieldPath, schema, defaultVal);
                    }
                }
            } else {
                // 添加新条目，创建键名输入框和值字段
                // 使用特殊的 name 属性格式：path[__key__] 用于标识这是键名输入框
                // 添加 oninput 事件，当用户输入键名时立即更新相关字段的 name 属性
                fieldHTML = '<input type="text" class="dict-key-input" name="' + path + '[__key__]' + '" placeholder="输入键名..." value="" oninput="updateDictKey(\'' + path + '\', \'' + itemId + '\', this.value)">';

                // 根据值类型创建不同的字段
                // 使用临时键名 __temp__，在收集数据时会根据实际的键名输入框的值来替换
                if (schema.type === 'object' && schema.properties) {
                    // 对象类型：遍历所有属性
                    const valuePath = [path + '[__temp__]'];
                    for (const propName in schema.properties || {}) {
                        const propSchema = schema.properties[propName];
                        const propMeta = {
                            default: propSchema.default,
                            description: propSchema.description,
                            enum: propSchema.enum
                        };
                        const propRequired = (schema.required || []).includes(propName);
                        fieldHTML += createFieldHTML(propName, propSchema, propMeta, propRequired, valuePath);
                    }
                } else {
                    // 简单类型：直接创建输入框
                    const fieldPath = path + '[__temp__]';
                    if (schema.type === 'boolean') {
                        fieldHTML += createSwitchHTML(fieldPath, schema.default);
                    } else if (schema.type === 'number' || schema.type === 'integer') {
                        fieldHTML += createNumberInputHTML(fieldPath, schema, schema.default);
                    } else {
                        fieldHTML += createTextInputHTML(fieldPath, schema, schema.default || '');
                    }
                }
                fieldHTML += '<button type="button" class="btn btn-danger btn-sm btn-icon dict-item-remove" onclick="removeDictItem(\'' + path + '\', \'' + itemId + '\')" style="margin-top: 12px;">×</button>';
            }

            itemDiv.innerHTML = fieldHTML;
            container.appendChild(itemDiv);

            if (!existingKey) {
                // 新条目自动聚焦到键名输入框
                const keyInput = itemDiv.querySelector('.dict-key-input');
                if (keyInput) {
                    keyInput.focus();
                }
            }
        }

        function updateDictKey(path, itemId, newKey) {
            const container = document.getElementById('dict-items-' + path);
            if (!container) return;

            const item = container.querySelector('[data-id="' + itemId + '"]');
            if (!item) return;

            // 获取当前存储的键名，如果没有则默认为 __temp__
            const currentKey = item.dataset.currentKey || '__temp__';
            
            // 更新所有值字段的 name 属性
            const valueFields = item.querySelectorAll('[name]');
            if (newKey) {
                valueFields.forEach(function(valueField) {
                    const oldName = valueField.getAttribute('name');
                    // 跳过键名输入框本身
                    if (oldName === path + '[__key__]') {
                        return;
                    }
                    // 匹配模式：path[currentKey] 或 path[currentKey].subfield 或 path[currentKey][subkey]
                    // 使用当前存储的键名进行匹配
                    const keyPattern = new RegExp('^' + escapeRegExp(path) + '\\[' + escapeRegExp(currentKey) + '\\](.*)$');
                    const match = oldName.match(keyPattern);
                    if (match) {
                        const suffix = match[1];
                        valueField.setAttribute('name', path + '[' + newKey + ']' + suffix);
                    }
                });
                // 更新存储的当前键名
                item.dataset.currentKey = newKey;
            }
        }

        function removeDictItem(path, itemId) {
            const container = document.getElementById('dict-items-' + path);
            if (!container) return;

            const item = container.querySelector('[data-id="' + itemId + '"]');
            if (item) {
                item.remove();
            }
        }

        function initializeFormValues() {
            document.querySelectorAll('.switch input').forEach(function(switchEl) {
                updateSwitchText(switchEl);
                switchEl.addEventListener('change', function(e) { updateSwitchText(e.target); });
            });
        }

        function updateSwitchText(checkbox) {
            const container = checkbox.closest('.switch-container');
            const textSpan = container.querySelector('span');
            if (textSpan) {
                textSpan.textContent = checkbox.checked ? '已启用' : '已禁用';
            }
        }

        function escapeHtml(text) {
            const div = document.createElement('div');
            div.textContent = text;
            return div.innerHTML;
        }
        
        function toggleCollapse(element) {
            const section = element.closest('.form-section');
            if (section) {
                // 获取折叠按钮
                const toggleBtn = element.classList.contains('collapse-toggle') ? element : section.querySelector('.collapse-toggle');
                
                // 检查当前是否是折叠状态
                const isCurrentlyCollapsed = section.classList.contains('collapsed');
                
                if (isCurrentlyCollapsed) {
                    // 当前是折叠状态，需要展开
                    section.classList.remove('collapsed');
                    
                    // 确保移除了类，再设置按钮文本
                    setTimeout(() => {
                        if (toggleBtn) {
                            toggleBtn.textContent = '▼';
                        }
                    }, 0);
                } else {
                    // 当前是展开状态，需要折叠
                    section.classList.add('collapsed');
                    
                    // 确保添加了类，再设置按钮文本
                    setTimeout(() => {
                        if (toggleBtn) {
                            toggleBtn.textContent = '▶';
                        }
                    }, 0);
                }
                
                // 强制重排以确保CSS变更被应用
                void section.offsetWidth;
            }
        }

        function escapeRegExp(str) {
            return str.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
        }

        function clearErrors() {
            document.querySelectorAll('.has-error').forEach(function(el) { el.classList.remove('has-error'); });
            document.querySelectorAll('.field-error').forEach(function(el) { el.textContent = ''; });
        }

        function showError(path, message) {
            const fieldGroup = document.querySelector('[data-path="' + path + '"]');
            if (fieldGroup) {
                fieldGroup.classList.add('has-error');
                const errorDiv = document.getElementById('error-' + path);
                if (errorDiv) {
                    errorDiv.textContent = message;
                }
            }
        }

        async function handleSubmit(event) {
            event.preventDefault();
            
            clearErrors();
            const config = {};
            
            const formData = new FormData(event.target);
            
            for (const name in schemaData.properties) {
                const prop = schemaData.properties[name];
                const value = collectFieldValue(name, prop, formData);
                if (value !== undefined) {
                    config[name] = value;
                }
            }
            
            for (let i = 0; i < schemaData.required.length; i++) {
                const required = schemaData.required[i];
                if (config[required] === undefined || config[required] === '' || config[required] === null) {
                    showError(required, '此字段为必填项');
                    return;
                }
            }

            try {
                const response = await fetch('/api/save', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ config: config })
                });

                const result = await response.json();

                if (result.success) {
                    // 弹出成功提示
                    alert('✅ ' + result.message + '\n\n页面即将关闭...');
                    
                    // 调用 shutdown 接口关闭服务器
                    fetch('/api/shutdown', { method: 'POST' }).catch(() => {});
                    
                    // 关闭页面
                    setTimeout(function() {
                        window.close();
                        // 如果 window.close() 不起作用（非脚本打开的窗口），显示提示
                        document.body.innerHTML = '<div style="display:flex;justify-content:center;align-items:center;height:100vh;font-size:24px;color:#059669;">✅ 配置已保存，请手动关闭此页面</div>';
                    }, 500);
                } else {
                    for (let i = 0; i < result.errors.length; i++) {
                        const err = result.errors[i];
                        showError(err.path, err.message);
                    }
                }
            } catch (error) {
                document.getElementById('error').style.display = 'block';
                document.getElementById('error').textContent = '请求失败：' + error.message;
            }
        }

        // 辅助函数：将路径字符串解析为嵌套对象并设置值
        function setNestedValue(obj, path, value) {
            // 解析路径，支持 .field 和 [key] 格式
            const parts = [];
            let remaining = path;
            while (remaining) {
                if (remaining.startsWith('.')) {
                    // 处理 .field 格式
                    remaining = remaining.substring(1);
                    const dotMatch = remaining.match(/^([^.\[]+)/);
                    if (dotMatch) {
                        parts.push(dotMatch[1]);
                        remaining = remaining.substring(dotMatch[1].length);
                    }
                } else if (remaining.startsWith('[')) {
                    // 处理 [key] 格式
                    const bracketMatch = remaining.match(/^\[([^\]]+)\]/);
                    if (bracketMatch) {
                        parts.push(bracketMatch[1]);
                        remaining = remaining.substring(bracketMatch[0].length);
                    } else {
                        break;
                    }
                } else {
                    break;
                }
            }
            
            // 设置嵌套值
            let current = obj;
            for (let i = 0; i < parts.length - 1; i++) {
                if (!current[parts[i]]) {
                    current[parts[i]] = {};
                }
                current = current[parts[i]];
            }
            if (parts.length > 0) {
                current[parts[parts.length - 1]] = value;
            }
        }

        // 辅助函数：递归处理嵌套对象中的 __key__ 和 __temp__ 键
        function processNestedDictKeys(obj) {
            if (typeof obj !== 'object' || obj === null) {
                return obj;
            }
            
            if (Array.isArray(obj)) {
                return obj.map(item => processNestedDictKeys(item));
            }
            
            const result = {};
            let tempValue = null;
            let actualKey = null;
            
            // 第一遍：收集 __key__ 和 __temp__ 的值
            for (const key in obj) {
                if (key === '__key__') {
                    actualKey = obj[key];
                } else if (key === '__temp__') {
                    tempValue = processNestedDictKeys(obj[key]);
                } else {
                    // 递归处理其他键的值
                    result[key] = processNestedDictKeys(obj[key]);
                }
            }
            
            // 如果存在 __key__ 和 __temp__，用实际键名替换
            if (actualKey && tempValue !== null) {
                result[actualKey] = tempValue;
            } else if (tempValue !== null && !actualKey) {
                // 如果只有 __temp__ 没有 __key__，保留 __temp__ 的值但记录警告
                // 这种情况不应该发生，但为了安全起见处理它
                result['__temp__'] = tempValue;
            }
            
            return result;
        }

        function collectFieldValue(name, prop, formData) {
            const type = prop.type;

            // 先判断是否是字典类型（object 且有 additionalProperties 但没有 properties）
            if (type === 'object' && prop.additionalProperties && !prop.properties) {
                const dict = {};
                const tempData = {};  // 存储临时键名 __temp__ 的数据
                let actualKeyName = null;  // 存储用户输入的实际键名
                const entries = formData.entries();

                for (const [key, value] of entries) {
                    const match = key.match(new RegExp('^' + escapeRegExp(name) + '\\[([^\\]]+)\\](.*)$'));
                    if (match) {
                        const dictKey = match[1];
                        const subPath = match[2];

                        // 收集 __key__ 键的值（用户输入的实际键名）
                        if (dictKey === '__key__') {
                            if (value && value.trim() !== '') {
                                actualKeyName = value.trim();
                            }
                            continue;
                        }

                        // 如果是 __temp__ 键，先存储到 tempData 中
                        if (dictKey === '__temp__') {
                            if (!tempData.__temp__) {
                                tempData.__temp__ = subPath ? {} : value;
                            }
                            if (subPath) {
                                setNestedValue(tempData.__temp__, subPath, value);
                            } else {
                                tempData.__temp__ = value;
                            }
                            continue;
                        }

                        // 正常的字典键
                        if (!dict[dictKey]) {
                            dict[dictKey] = subPath ? {} : value;
                        }

                        if (subPath) {
                            // 使用辅助函数处理深层嵌套路径
                            setNestedValue(dict[dictKey], subPath, value);
                        } else {
                            dict[dictKey] = value;
                        }
                    }
                }

                // 处理临时键名 __temp__ 的数据
                // 使用第一次遍历时收集的实际键名
                if (tempData.__temp__ !== undefined && actualKeyName) {
                    if (!dict[actualKeyName]) {
                        dict[actualKeyName] = tempData.__temp__;
                    } else {
                        // 如果该键已存在，合并数据
                        if (typeof dict[actualKeyName] === 'object' && typeof tempData.__temp__ === 'object') {
                            Object.assign(dict[actualKeyName], tempData.__temp__);
                        } else {
                            dict[actualKeyName] = tempData.__temp__;
                        }
                    }
                }

                // 类型转换
                const valueSchema = prop.additionalProperties || {};
                if (valueSchema.type === 'number' || valueSchema.type === 'integer') {
                    for (const key in dict) {
                        dict[key] = valueSchema.type === 'integer' ? parseInt(dict[key]) : parseFloat(dict[key]);
                    }
                } else if (valueSchema.type === 'boolean') {
                    for (const key in dict) {
                        dict[key] = dict[key] === 'true';
                    }
                }

                // 递归处理嵌套对象中的 __key__ 和 __temp__ 键
                const processedDict = processNestedDictKeys(dict);
                return Object.keys(processedDict).length > 0 ? processedDict : prop.default || {};
            }
            
            if (type === 'array') {
                const values = [];
                const entries = formData.entries();
                let index = 0;
                
                for (const [key, value] of entries) {
                    if (key.startsWith(name + '[')) {
                        const match = key.match(new RegExp('^' + escapeRegExp(name) + '\\[(\\d+)\\](.*)$'));
                        if (match) {
                            const idx = parseInt(match[1]);
                            const subPath = match[2];
                            
                            while (values.length <= idx) {
                                values.push(subPath ? {} : null);
                            }
                            
                            if (subPath) {
                                if (subPath.startsWith('.')) {
                                    const subField = subPath.substring(1);
                                    if (!values[idx]) values[idx] = {};
                                    values[idx][subField] = value;
                                } else {
                                    values[idx][subPath] = value;
                                }
                            } else {
                                values[idx] = value;
                            }
                        }
                    }
                }
                
                if (values.length === 0) {
                    return prop.default || [];
                }
                
                const itemSchema = prop.items || {};
                if (itemSchema.type === 'number' || itemSchema.type === 'integer') {
                    return values.map(v => v ? (itemSchema.type === 'integer' ? parseInt(v) : parseFloat(v)) : v).filter(v => v !== null && v !== '');
                } else if (itemSchema.type === 'boolean') {
                    return values.map(v => v === 'true').filter(v => v !== null && v !== '');
                }
                
                return values.filter(v => v !== null && v !== '');
            }
            
            if (type === 'object') {
                const obj = {};
                const entries = formData.entries();
                const prefix = name + '.';
                
                for (const [key, value] of entries) {
                    if (key.startsWith(prefix)) {
                        const subField = key.substring(prefix.length);
                        if (prop.properties && prop.properties[subField]) {
                            const subType = prop.properties[subField].type;
                            if (subType === 'boolean') {
                                obj[subField] = value === 'true';
                            } else if (subType === 'number' || subType === 'integer') {
                                obj[subField] = subType === 'integer' ? parseInt(value) : parseFloat(value);
                            } else {
                                obj[subField] = value;
                            }
                        } else {
                            obj[subField] = value;
                        }
                    }
                }
                
                return Object.keys(obj).length > 0 ? obj : prop.default || {};
            }
            
            const value = formData.get(name);

            if (value === null || value === '') {
                if (type === 'boolean') {
                    return false;  // checkbox 未选中时返回 false，而不是默认值
                }
                return prop.default;
            }

            if (type === 'boolean') {
                return value === 'true';
            } else if (type === 'number' || type === 'integer') {
                return type === 'integer' ? parseInt(value) : parseFloat(value);
            } else {
                return value;
            }
        }

        // 使用事件委托处理折叠操作
        document.getElementById('form-fields').addEventListener('click', function(e) {
            if (e.target.classList.contains('form-section-header')) {
                // 如果点击的是折叠按钮，则不触发头部的折叠功能
                if (!e.target.classList.contains('collapse-toggle')) {
                    toggleCollapse(e.target);
                }
            } else if (e.target.classList.contains('collapse-toggle')) {
                e.stopPropagation();
                toggleCollapse(e.target.closest('.form-section'));
            }
        });
        
        document.getElementById('config-form').addEventListener('submit', handleSubmit);
        loadSchema();
    </script>
</body>
</html>
"""



"""LLM 模块规划 Agent 的执行器。"""

import os
import subprocess

from jarvis.jarvis_utils.output import PrettyOutput

# -*- coding: utf-8 -*-
from pathlib import Path
from typing import List
from typing import Optional
from typing import Union

from jarvis.jarvis_c2rust.llm_module_agent_apply import (
    apply_project_structure_from_json,
)
from jarvis.jarvis_c2rust.llm_module_agent_utils import parse_project_json_entries
from jarvis.jarvis_c2rust.llm_module_agent_utils import resolve_created_dir


def execute_llm_plan(
    out: Optional[Union[Path, str]] = None,
    apply: bool = False,
    crate_name: Optional[Union[Path, str]] = None,
    llm_group: Optional[str] = None,
    non_interactive: bool = True,
    enable_ffi_export_validation: bool = False,
) -> List:
    """
    返回 LLM 生成的目录结构原始 JSON 文本（来自 <PROJECT> 块）。
    不进行解析，便于后续按原样应用并在需要时使用更健壮的解析器处理。
    """
    # execute_llm_plan 是顶层入口，需要执行清理（skip_cleanup=False）
    # 延迟导入以避免循环依赖
    from jarvis.jarvis_c2rust.llm_module_agent import plan_crate_json_text

    json_text = plan_crate_json_text(llm_group=llm_group, skip_cleanup=False)
    entries, parse_error = parse_project_json_entries(json_text)
    if parse_error:
        raise ValueError(f"JSON解析失败: {parse_error}")
    if not entries:
        raise ValueError("[c2rust-llm-planner] 从LLM输出解析目录结构失败。正在中止。")

    # 2) 如需应用到磁盘
    if apply:
        target_root = crate_name if crate_name else "."
        try:
            apply_project_structure_from_json(json_text, project_root=target_root)
            PrettyOutput.auto_print("[c2rust-llm-planner] 项目结构已应用。")
        except Exception as e:
            PrettyOutput.auto_print(f"[c2rust-llm-planner] 应用项目结构失败: {e}")
            raise

        # Post-apply: 检查生成的目录结构，使用 CodeAgent 更新 Cargo.toml
        from jarvis.jarvis_code_agent.code_agent import (
            CodeAgent,
        )  # 延迟导入以避免全局耦合

        # 解析 crate 目录路径（与 apply 逻辑保持一致）
        try:
            created_dir = resolve_created_dir(target_root)
        except Exception:
            # 兜底：无法解析时直接使用传入的 target_root
            created_dir = Path(target_root)

        # 在 crate 目录内执行 git 初始化与初始提交（按新策略）
        try:
            # 初始化 git 仓库（若已存在则该命令为幂等）
            subprocess.run(["git", "init"], check=False, cwd=str(created_dir))
            # 添加所有文件并尝试提交
            subprocess.run(["git", "add", "-A"], check=False, cwd=str(created_dir))
            subprocess.run(
                ["git", "commit", "-m", "[c2rust-llm-planner] init crate"],
                check=False,
                cwd=str(created_dir),
            )
        except Exception:
            # 保持稳健，不因 git 失败影响主流程
            pass

        # 构建用于 CodeAgent 的目录上下文（简化版树形）
        def _format_tree(root: Path) -> str:
            lines: List[str] = []
            exclude = {".git", "target", ".jarvis"}
            if not root.exists():
                return ""
            for p in sorted(root.rglob("*")):
                if any(part in exclude for part in p.parts):
                    continue
                rel = p.relative_to(root)
                depth = len(rel.parts) - 1
                indent = "  " * depth
                name = rel.name + ("/" if p.is_dir() else "")
                lines.append(f"{indent}- {name}")
            return "\n".join(lines)

        dir_ctx = _format_tree(created_dir)
        crate_pkg_name = created_dir.name

        requirement_lines = [
            "目标：在该 crate 目录下确保 `cargo build` 能成功完成；如失败则根据错误最小化修改并重试，直到构建通过为止。",
            f"- crate_dir: {created_dir}",
            f"- crate_name: {crate_pkg_name}",
            "目录结构（部分）：",
            dir_ctx,
            "",
            "执行与修复流程（务必按序执行，可多轮迭代）：",
            "1) 先补齐 Rust 模块声明（仅最小化追加/升级，不覆盖业务实现）：",
            "   - 扫描 src 目录：",
            "     * 在每个子目录下（除 src 根）创建或更新 mod.rs，仅追加缺失的 `pub mod <child>;` 声明；",
            "     * 在 src/lib.rs 中为顶级子模块追加 `pub mod <name>;`；不要创建 src/mod.rs；忽略 lib.rs 与 main.rs 的自引用；",
            "   - 若存在 `mod <name>;` 但非 pub，则就地升级为 `pub mod <name>;`，保留原缩进与其他内容；",
            "   - 严禁删除现有声明或修改非声明代码；",
            '2) 在 Cargo.toml 的 [package] 中设置 edition："2024"；若本地工具链不支持 2024，请降级为 "2021" 并在说明中记录原因；保留其他已有字段与依赖不变。',
            "3) 根据当前源代码实际情况配置入口：",
            "   - 仅库：仅配置 [lib]（path=src/lib.rs），不要生成 main.rs；"
            + (
                '**重要**：必须在 [lib] 配置中添加 `crate-type = ["cdylib", "rlib"]`（或至少包含 "cdylib"），确保产物包含 cdylib。可以同时包含 rlib，但不能只有 rlib。'
                if enable_ffi_export_validation
                else ""
            ),
            "   - 单一可执行：存在 src/main.rs 时配置 [[bin]] 或默认二进制；可选保留 [lib] 以沉淀共享逻辑；"
            + (
                '**重要**：如果配置了 [lib]，必须在 [lib] 配置中添加 `crate-type = ["cdylib", "rlib"]`（或至少包含 "cdylib"），确保产物包含 cdylib。可以同时包含 rlib，但不能只有 rlib。'
                if enable_ffi_export_validation
                else ""
            ),
            "   - 多可执行：为每个 src/bin/<name>.rs 配置 [[bin]]；共享代码放在 src/lib.rs；"
            + (
                '**重要**：必须在 [lib] 配置中添加 `crate-type = ["cdylib", "rlib"]`（或至少包含 "cdylib"），确保产物包含 cdylib。可以同时包含 rlib，但不能只有 rlib。'
                if enable_ffi_export_validation
                else ""
            ),
            "   - 不要创建与目录结构不一致的占位入口。",
            "4) 对被作为入口的源文件：若不存在 fn main() 则仅添加最小可用实现（不要改动已存在的实现）：",
            '   fn main() { println!("ok"); }',
            "5) 执行一次构建验证：`cargo build -q`（或 `cargo check -q`）。",
            "6) 若构建失败，读取错误并进行最小化修复，然后再次构建；重复直至成功。仅允许的修复类型：",
            "   - 依赖缺失：在 [dependencies] 中添加必要且稳定版本的依赖（优先无特性），避免新增未使用依赖；",
            "   - 入口/crate-type 配置错误：修正 [lib] 或 [[bin]] 的 name/path/crate-type 使之与目录与入口文件一致；",
            "   - 语言/工具链不兼容：将 edition 从 2024 调整为 2021；必要时可添加 rust-version 要求；",
            "   - 语法级/最小实现缺失：仅在入口文件中补充必要的 use/空实现/feature gate 以通过编译，避免改动非入口业务文件；",
            "   - 不要删除或移动现有文件与目录。",
            "7) 每轮修改后必须运行 `cargo build -q` 验证，直到构建成功为止。",
            "",
            "修改约束：",
            "- 允许修改的文件范围：Cargo.toml、src/lib.rs、src/main.rs、src/bin/*.rs、src/**/mod.rs（仅最小必要变更）；除非为修复构建与模块声明补齐，不要修改其他文件。",
            "- 尽量保持现有内容与结构不变，不要引入与构建无关的改动或格式化。",
            "",
            "交付要求：",
            "- 以补丁方式提交实际修改的文件；",
            "- 在最终回复中简要说明所做变更与最终 `cargo build` 的结果（成功/失败及原因）。",
        ]
        requirement_text = "\n".join(requirement_lines)

        prev_cwd = os.getcwd()
        try:
            # 切换到 crate 目录运行 CodeAgent 与构建
            os.chdir(str(created_dir))
            PrettyOutput.auto_print(
                f"[c2rust-llm-planner] 已切换到 crate 目录: {os.getcwd()}，执行 CodeAgent 初始化"
            )
            if llm_group:
                PrettyOutput.auto_print(f"[c2rust-llm-planner] 使用模型组: {llm_group}")
            try:
                # 验证模型配置在切换目录后是否仍然有效
                # CodeAgent 使用 smart 模型，所以这里也使用 smart 配置来显示正确的模型信息
                from jarvis.jarvis_utils.config import get_smart_model_name
                from jarvis.jarvis_utils.config import get_smart_platform_name

                if llm_group:
                    resolved_model = get_smart_model_name(llm_group)
                    resolved_platform = get_smart_platform_name(llm_group)
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner] 解析的模型配置: 平台={resolved_platform}, 模型={resolved_model}"
                    )
            except Exception as e:
                PrettyOutput.auto_print(
                    f"[c2rust-llm-planner] 警告: 无法验证模型配置: {e}"
                )

            try:
                agent = CodeAgent(
                    need_summary=False,
                    non_interactive=non_interactive,
                    model_group=llm_group,
                    enable_task_list_manager=False,
                    disable_review=True,
                )
                # 验证 agent 内部的模型配置
                if hasattr(agent, "model") and agent.model:
                    actual_model = getattr(agent.model, "model_name", "unknown")
                    actual_platform = type(agent.model).__name__
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner] CodeAgent 内部模型: {actual_platform}.{actual_model}"
                    )
                agent.run(requirement_text, prefix="[c2rust-llm-planner]", suffix="")
                PrettyOutput.auto_print(
                    "[c2rust-llm-planner] 初始 CodeAgent 运行完成。"
                )
            except Exception as e:
                error_msg = str(e)
                if "does not exist" in error_msg or "404" in error_msg:
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner] 模型配置错误: {error_msg}"
                    )
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner] 提示: 请检查模型组 '{llm_group}' 的配置是否正确"
                    )
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner] 当前工作目录: {os.getcwd()}"
                    )
                    # 尝试显示当前解析的模型配置（CodeAgent 使用 smart 模型）
                    try:
                        from jarvis.jarvis_utils.config import get_smart_model_name
                        from jarvis.jarvis_utils.config import get_smart_platform_name

                        if llm_group:
                            PrettyOutput.auto_print(
                                f"[c2rust-llm-planner] 当前解析的模型: {get_smart_platform_name(llm_group)}/{get_smart_model_name(llm_group)}"
                            )
                    except Exception:
                        pass
                raise

            # 进入构建与修复循环：构建失败则生成新的 CodeAgent，携带错误上下文进行最小修复
            iter_count = 0
            while True:
                iter_count += 1
                PrettyOutput.auto_print(
                    f"[c2rust-llm-planner] 在 {os.getcwd()} 执行: cargo build -q"
                )
                build_res = subprocess.run(
                    ["cargo", "build", "-q"],
                    capture_output=True,
                    text=True,
                    check=False,
                )
                stdout = build_res.stdout or ""
                stderr = build_res.stderr or ""
                output = (stdout + "\n" + stderr).strip()

                if build_res.returncode == 0:
                    PrettyOutput.auto_print("[c2rust-llm-planner] Cargo 构建成功。")
                    break

                PrettyOutput.auto_print(
                    f"[c2rust-llm-planner] Cargo 构建失败 (iter={iter_count})。"
                )
                # 打印编译错误输出，便于可视化与调试
                PrettyOutput.auto_print("[c2rust-llm-planner] 构建错误输出:")
                PrettyOutput.auto_print(output)
                # 将错误信息作为上下文，附加修复原则，生成新的 CodeAgent 进行最小修复
                repair_prompt = "\n".join(
                    [
                        requirement_text,
                        "",
                        "请根据以下构建错误进行最小化修复，然后再次执行 `cargo build` 验证：",
                        "<BUILD_ERROR>",
                        output,
                        "</BUILD_ERROR>",
                    ]
                )

                if llm_group:
                    PrettyOutput.auto_print(
                        f"[c2rust-llm-planner][iter={iter_count}] 使用模型组: {llm_group}"
                    )
                try:
                    repair_agent = CodeAgent(
                        need_summary=False,
                        non_interactive=non_interactive,
                        model_group=llm_group,
                        enable_task_list_manager=False,
                        disable_review=True,
                    )
                    repair_agent.run(
                        repair_prompt,
                        prefix=f"[c2rust-llm-planner][iter={iter_count}]",
                        suffix="",
                    )
                except Exception as e:
                    error_msg = str(e)
                    if "does not exist" in error_msg or "404" in error_msg:
                        PrettyOutput.auto_print(
                            f"[c2rust-llm-planner][iter={iter_count}] 模型配置错误: {error_msg}"
                        )
                        PrettyOutput.auto_print(
                            f"[c2rust-llm-planner][iter={iter_count}] 提示: 请检查模型组 '{llm_group}' 的配置"
                        )
                    raise
                # 不切换目录，保持在原始工作目录
        finally:
            # 恢复之前的工作目录
            os.chdir(prev_cwd)

    # 3) 输出 JSON 到文件（如指定），并返回解析后的 entries
    if out is not None:
        out_path = Path(out)
        out_path.parent.mkdir(parents=True, exist_ok=True)
        # 使用原始文本写出，便于可读
        out_path.write_text(json_text, encoding="utf-8")
        PrettyOutput.auto_print(f"[c2rust-llm-planner] JSON 已写入: {out_path}")

    return entries



# -*- coding: utf-8 -*-
import json
import os
from typing import Any
from typing import Dict
from typing import Optional
from typing import Generator
from typing import List
from typing import Tuple

from anthropic import Anthropic
from anthropic.types import MessageParam

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils.output import PrettyOutput


class ClaudeModel(BasePlatform):
    def __init__(self, llm_config: Optional[Dict[str, Any]] = None):
        """
        Initialize Claude model

        参数:
            llm_config: LLM配置字典，包含 anthropic_api_key, anthropic_base_url 等
        """
        super().__init__()
        self.system_message = ""
        llm_config = llm_config or {}

        # API Key 配置
        if llm_config:
            if "anthropic_api_key" in llm_config:
                self.api_key = llm_config.get("anthropic_api_key")
            else:
                self.api_key = os.getenv("ANTHROPIC_API_KEY")
        else:
            self.api_key = os.getenv("ANTHROPIC_API_KEY")

        # Base URL 配置
        if llm_config:
            if "anthropic_base_url" in llm_config:
                self.base_url = llm_config.get("anthropic_base_url")
            else:
                self.base_url = os.getenv("ANTHROPIC_BASE_URL")
        else:
            self.base_url = os.getenv("ANTHROPIC_BASE_URL")

        # API Key 警告
        if not self.api_key and llm_config:
            PrettyOutput.auto_print("⚠️ ANTHROPIC_API_KEY 未设置")

        # 模型名称
        self.model_name = os.getenv("model") or "claude-3-5-sonnet-20241022"

        # 初始化 Anthropic 客户端
        self.client = None
        try:
            if self.base_url:
                self.client = Anthropic(api_key=self.api_key, base_url=self.base_url)
            else:
                self.client = Anthropic(api_key=self.api_key)
        except Exception as e:
            PrettyOutput.auto_print(f"⚠️ 初始化 Anthropic 客户端失败: {e}")

        # 消息历史
        self.messages: List[Dict[str, str]] = []
        self.system_message = ""

    def get_model_list(self) -> List[Tuple[str, str]]:
        """
        获取可用的 Claude 模型列表

        返回:
            List[Tuple[str, str]]: 模型ID和名称的元组列表

        异常:
            当API调用失败时会打印错误信息并返回空列表
        """
        if not self.client:
            PrettyOutput.auto_print("❌ Anthropic 客户端未初始化")
            return []

        try:
            # 尝试使用models API获取实际的模型列表
            model_response = self.client.models.list()
            models = []
            for model in model_response.data:
                model_id = model.id if hasattr(model, "id") else str(model)
                model_name = model.id if hasattr(model, "id") else str(model)
                models.append((model_id, model_name))

            if models:
                PrettyOutput.auto_print("✅ 成功从API获取模型列表")
                return models
            else:
                PrettyOutput.auto_print("⚠️ API响应中没有模型数据")
                return []
        except AttributeError:
            # models API 不存在或不支持
            PrettyOutput.auto_print("❌ 模型列表API不可用")
            return []
        except Exception as e:
            PrettyOutput.auto_print(f"❌ 获取模型列表失败: {e}")
            return []

    def set_model_name(self, model_name: str):
        """
        设置当前使用的模型名称

        参数:
            model_name: 要设置的模型名称
        """
        self.model_name = model_name

    def set_system_prompt(self, message: str):
        """
        设置系统消息(角色设定)

        参数:
            message: 系统消息内容

        说明:
            Claude 使用单独的 system 参数，不是消息列表的一部分
        """
        self.system_message = message

    def chat(self, message: str) -> Generator[str, None, None]:
        """
        执行对话并返回生成器

        参数:
            message: 用户输入的消息内容

        返回:
            Generator[str, None, None]: 生成器，逐块返回AI响应内容

        异常:
            当API调用失败时会抛出异常并打印错误信息
        """
        if not self.client:
            PrettyOutput.auto_print("❌ Anthropic 客户端未初始化")
            raise Exception("Anthropic client not initialized")

        try:
            # 转换消息格式为 Anthropic 格式
            anthropic_messages: List[MessageParam] = []
            for msg in self.messages:
                role = msg.get("role")
                content = msg.get("content")
                if role == "user" and content:
                    anthropic_messages.append({"role": "user", "content": content})
                elif role == "assistant" and content:
                    anthropic_messages.append({"role": "assistant", "content": content})

            # 添加当前用户消息
            anthropic_messages.append({"role": "user", "content": message})

            # 累计完整响应
            accumulated_response = ""

            # 准备 system 参数：后端期望数组格式，而不是字符串
            system_param = None
            if self.system_message:
                # 将字符串转换为数组格式：[{"type": "text", "text": "系统消息内容"}]
                system_param = [{"type": "text", "text": self.system_message}]

            # 调用 Anthropic API
            from typing import Union

            stream_kwargs: Dict[
                str,
                Union[
                    str,
                    List[MessageParam],
                    int,
                    List[List[Dict[str, str]]],
                    List[Dict[str, str]],
                ],
            ] = {
                "model": self.model_name,
                "messages": anthropic_messages,
                "max_tokens": 4096,
            }
            if system_param:
                stream_kwargs["system"] = system_param

            with self.client.messages.stream(**stream_kwargs) as stream:  # type: ignore
                full_response = ""
                for text in stream.text_stream:
                    full_response += text
                    accumulated_response += text
                    yield text

            # 将响应添加到消息历史
            if accumulated_response:
                self.messages.append({"role": "user", "content": message})
                self.messages.append(
                    {"role": "assistant", "content": accumulated_response}
                )

            return None

        except Exception as e:
            PrettyOutput.auto_print(f"❌ 对话失败：{str(e)}")
            raise Exception(f"Chat failed: {str(e)}")

    def name(self) -> str:
        """
        获取当前使用的模型名称

        返回:
            str: 当前配置的模型名称
        """
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """
        获取当前平台的名称

        返回:
            str: 当前平台的名称
        """
        return "claude"

    def delete_chat(self) -> bool:
        """
        删除当前对话历史

        返回:
            bool: 操作是否成功

        说明:
            系统消息会保留在 system_message 中
        """
        self.messages = []
        return True

    def save(self, file_path: str) -> bool:
        """
        Save chat session to a file.

        参数:
            file_path: 保存会话文件的路径

        返回:
            bool: 保存是否成功
        """
        state: Dict[str, Any] = {
            "messages": self.messages,
            "model_name": self.model_name,
            "system_message": self.system_message,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.auto_print(f"✅ 会话已成功保存到 {file_path}")
            return True
        except Exception as e:
            PrettyOutput.auto_print(f"❌ 保存会话失败: {str(e)}")
            return False

    def restore(self, file_path: str) -> bool:
        """
        Restore chat session from a file.

        参数:
            file_path: 要恢复会话文件的路径

        返回:
            bool: 恢复是否成功
        """
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.messages = state.get("messages", [])
            self.model_name = state.get("model_name", "claude-3-5-sonnet-20241022")
            self.system_message = state.get("system_message", "")
            # 处理start_commit信息（如果存在）
            # start_commit = state.get("start_commit", None)
            # 可以根据需要使用start_commit信息
            self._saved = True

            PrettyOutput.auto_print(f"✅ 从 {file_path} 成功恢复会话")
            return True
        except FileNotFoundError:
            PrettyOutput.auto_print(f"❌ 会话文件未找到: {file_path}")
            return False
        except Exception as e:
            PrettyOutput.auto_print(f"❌ 恢复会话失败: {str(e)}")
            return False

    def trim_messages(self) -> bool:
        """裁剪消息历史以腾出token空间

        Claude的system_message单独存储，不在messages列表中。
        丢弃开头的10条非system消息（user/assistant消息）。

        返回:
            bool: 如果成功腾出空间返回True，否则返回False
        """
        if not self.messages:
            return False

        # Claude的messages列表只包含user和assistant消息
        # 如果消息少于等于10条，无法裁剪
        if len(self.messages) <= 10:
            PrettyOutput.auto_print("⚠️ 警告：消息不足10条，无法裁剪")
            return False

        # 记录原始消息数量
        original_count = len(self.messages)

        # 丢弃开头的10条消息
        self.messages = self.messages[10:]
        trimmed_count = original_count - len(self.messages)

        # 检查裁剪后是否有剩余token
        remaining_tokens = self.get_remaining_token_count()
        if remaining_tokens > 0:
            PrettyOutput.auto_print(
                f"✅ 裁剪成功：丢弃了{trimmed_count}条消息，剩余token: {remaining_tokens}"
            )
            return True
        else:
            PrettyOutput.auto_print(
                f"⚠️ 警告：已裁剪{trimmed_count}条消息，但仍无剩余token"
            )
            return False

    @classmethod
    def get_required_env_keys(cls) -> List[str]:
        """
        获取Claude平台所需的配置键列表

        返回:
            List[str]: 配置键的列表（对应 llm_config 中的 anthropic_api_key）
        """
        return ["ANTHROPIC_API_KEY"]

    @classmethod
    def get_env_config_guide(cls) -> Dict[str, str]:
        """
        获取配置指导

        返回:
            Dict[str, str]: 配置键名到配置指导的映射
        """
        return {
            "ANTHROPIC_API_KEY": (
                "请输入您的 Anthropic API Key:\n"
                "获取方式:\n"
                "1. 登录 Anthropic 控制台: https://console.anthropic.com/\n"
                "2. 进入 API Keys 页面\n"
                "3. 创建新的 API Key\n"
                "4. 复制 API Key"
            ),
        }

    @classmethod
    def get_env_defaults(cls) -> Dict[str, str]:
        """
        获取Claude平台环境变量的默认值

        返回:
            Dict[str, str]: 环境变量默认值的字典
        """
        return {}
